
from test_base import TestBase
import pywbem
import time
import random

class TestBriding(TestBase):
    def setUp(self):
        TestBase.setUp(self)
        if not self.fake_test and len(self.ports) < 2:
            self.skipTest("Specify at least two ports to test bridging")

        self.confService = self.wbemconnection.EnumerateInstances("LMI_IPConfigurationService")[0]

        self.networkConnections = []
        les = self.wbemconnection.EnumerateInstances("LMI_IPNetworkConnection")
        for le in les:
            if le["Name"] in self.ports and le["Name"] != "lo":
                self.networkConnections.append(le)
        print "Running test on: ", [le["Name"] for le in self.networkConnections]
        self.assertGreater(len(self.networkConnections), 1, "At least two ports required for bridging test")

        # Subscribe to indication
        self.filter_name = "test_apply_%d" % (time.time() * 1000)
        self.subscribe(self.filter_name, "select * from LMI_NetworkInstModification where SourceInstance isa LMI_NetworkJob")

    def test_createBridge(self):
        # Create Bridge connection
        caps = self.wbemconnection.Associators(self.networkConnections[0].path, ResultClass="LMI_IPNetworkConnectionCapabilities")
        self.assertEqual(len(caps), 1)
        cap = caps[0]

        caption = "XXX Test DHCP Bridge"
        rc = self.wbemconnection.InvokeMethod("LMI_CreateIPSetting", cap.path,
                Caption=caption,
                Type=pywbem.Uint16(5), # Bridging
                IPv4Type=pywbem.Uint16(4), # DHCP
                IPv6Type=pywbem.Uint16(9)) # Stateless

        self.assertEqual(len(rc), 2)
        self.assertEqual(rc[0], 0)
        self.assertTrue(rc[1] is not None)
        settingData = self.wbemconnection.GetInstance(rc[1]["SettingData"])

        slaves = self.wbemconnection.AssociatorNames(settingData.path,
                ResultClass="LMI_BridgingSlaveSettingData",
                AssocClass="LMI_OrderedIPAssignmentComponent")
        self.assertEqual(len(slaves), 1)

        # Save the slave SettingData
        slaveSettingDataList = {}
        slaveSettingDataList[self.networkConnections[0]] = slaves[0]

        # Delete the connection on cleanup
        # Deleting master connection will also delete the slaves
        self.addCleanup(self.wbemconnection.DeleteInstance, settingData.path)

        # Check object presence
        for sd in self.wbemconnection.EnumerateInstances("LMI_BridgingMasterSettingData"):
            if sd["Caption"] == caption:
                break
        else:
            self.assertTrue(False, "No instance of LMI_IPAssignmentSettingData with Caption \"%s\"" % caption)

        # Check if properties are present
        self.assertTrue(settingData["AgeingTime"] is not None)
        self.assertTrue(settingData["ForwardDelay"] is not None)
        self.assertTrue(settingData["HelloTime"] is not None)
        self.assertTrue(settingData["InterfaceName"] is not None)
        self.assertTrue(settingData["MaxAge"] is not None)
        self.assertTrue(settingData["Priority"] is not None)
        self.assertTrue(settingData["STP"] is not None)

        # Modify some properties
        settingData["AgeingTime"] = pywbem.Uint32(301)
        settingData["ForwardDelay"] =  pywbem.Uint32(16)
        settingData["HelloTime"] = pywbem.Uint32(3)
        # Use random name to minimize probability to reuse existing port name
        bridgeName = "bridge%d" % random.randint(0, 100)
        settingData["InterfaceName"] = bridgeName
        settingData["MaxAge"] = pywbem.Uint32(21)
        settingData["Priority"] = pywbem.Uint32(128)
        self.wbemconnection.ModifyInstance(settingData)
        settingData = self.wbemconnection.GetInstance(settingData.path)

        # Check if the properties are modified
        self.assertEqual(settingData["AgeingTime"], 301)
        self.assertEqual(settingData["ForwardDelay"], 16)
        self.assertEqual(settingData["HelloTime"], 3)
        self.assertEqual(settingData["InterfaceName"], bridgeName)
        self.assertEqual(settingData["MaxAge"], 21)
        self.assertEqual(settingData["Priority"], 128)
        self.assertEqual(settingData["STP"], True)

        # Add slaves, first one is added automatically when creating IP setting
        for networkConnection in self.networkConnections[1:]:
            caps = self.wbemconnection.Associators(networkConnection.path, ResultClass="LMI_IPNetworkConnectionCapabilities")
            self.assertEqual(len(caps), 1)
            cap = caps[0]

            rc = self.wbemconnection.InvokeMethod("LMI_CreateSlaveSetting", cap.path, MasterSettingData=settingData.path)
            self.assertEqual(len(rc), 2)
            self.assertEqual(rc[0], 0)
            self.assertEqual(rc[1]["SettingData"].classname, "LMI_BridgingSlaveSettingData")
            slaveSettingDataList[networkConnection] = rc[1]["SettingData"]

        # Check if the slave SettingDatas are associated to master
        assoc = self.wbemconnection.Associators(settingData.path, ResultClass="LMI_BridgingSlaveSettingData")
        self.assertEqual(len(assoc), len(self.networkConnections))

        # Test modification of slave setting data
        slaveSettingData = assoc[0]
        slaveSettingData["Priority"] = pywbem.Uint32(33)
        slaveSettingData["PathCost"] = pywbem.Uint32(101)
        slaveSettingData["HairpinMode"] = True
        self.wbemconnection.ModifyInstance(slaveSettingData)
        slaveSettingData = self.wbemconnection.GetInstance(slaveSettingData.path)

        # Check if the properties are modified
        self.assertEqual(slaveSettingData["Priority"], 33)
        self.assertEqual(slaveSettingData["PathCost"], 101)
        self.assertEqual(slaveSettingData["HairpinMode"], True)

        # Activate connection using SlaveSettingData
        for networkConnection in self.networkConnections:
            rc = self.wbemconnection.InvokeMethod("ApplySettingToIPNetworkConnection", self.confService.path,
                    SettingData=slaveSettingDataList[networkConnection], IPNetworkConnection=networkConnection.path, Mode=pywbem.Uint16(32768))
            self.assertIn(rc[0], [0, 4096])
            if rc[0] == 4096: # Job started
                # Wait for job completed indication
                self.wait_for_job_completion(rc[1]["Job"])

        # Check presence of SwitchService
        switchServices = self.wbemconnection.Associators(settingData.path, ResultClass="LMI_SwitchService",
                AssocClass="LMI_IPElementSettingData")
        self.assertGreater(len(switchServices), 0)
        switchService = switchServices[0]

        switchPorts = self.wbemconnection.EnumerateInstances("LMI_SwitchPort")
        self.assertEqual(len(switchPorts), 2)

        # Check presence of LMI_SwitchPort for each slave
        switchPortNames = [x["Name"] for x in switchPorts]
        for networkConnection in self.networkConnections:
            self.assertIn(networkConnection["Name"], switchPortNames)

        # Check if the slaves are associated with master
        aggregatedPorts = self.wbemconnection.Associators(switchService.path, ResultClass="LMI_SwitchPort",
                AssocClass="LMI_SwitchesAmong")
        self.assertEqual(len(aggregatedPorts), 2)
        for aggregatedPort in aggregatedPorts:
            self.assertIn(aggregatedPort["Name"], switchPortNames)

        # Check if slaves are identical to LANEndpoints
        for switchPort in switchPorts:
            lanendpoints = self.wbemconnection.Associators(switchPort.path, ResultClass="LMI_LANEndpoint",
                    AssocClass="LMI_EndpointIdentity")
            self.assertEqual(len(lanendpoints), 1)
            self.assertEqual(lanendpoints[0]["Name"], switchPort["Name"])

    @classmethod
    def setUpClass(cls):
        TestBase.setUpClass()

    @classmethod
    def tearDownClass(cls):
        TestBase.tearDownClass()
        # Give it some time to actually delete the connections
        time.sleep(1)
        connections = []
        for connection in cls.wbemconnection.EnumerateInstances("LMI_IPAssignmentSettingData"):
            if connection["Caption"].startswith("XXX "):
                connections.append(connection["Caption"])
        if len(connections) > 0:
            raise AssertionError("Connections was not deleted: %s" % ", ".join(connections))




from test_base import TestBase
import pywbem
import time

class TestBonding(TestBase):
    def setUp(self):
        TestBase.setUp(self)
        if not self.fake_test and len(self.ports) < 2:
            self.skipTest("Specify at least two ports to test bonding")

        self.confService = self.wbemconnection.EnumerateInstances("LMI_IPConfigurationService")[0]

        self.networkConnections = []
        les = self.wbemconnection.EnumerateInstances("LMI_IPNetworkConnection")
        for le in les:
            if le["Name"] in self.ports and le["Name"] != "lo":
                self.networkConnections.append(le)
        print "Running test on: ", [le["Name"] for le in self.networkConnections]
        self.assertGreater(len(self.networkConnections), 1, "At least two ports required for bonding test")

        # Subscribe to indication
        self.filter_name = "test_apply_%d" % (time.time() * 1000)
        self.subscribe(self.filter_name, "select * from LMI_NetworkInstModification where SourceInstance isa LMI_NetworkJob")

    def test_createBond(self):
        # Create Bond connection
        caps = self.wbemconnection.Associators(self.networkConnections[0].path, ResultClass="LMI_IPNetworkConnectionCapabilities")
        self.assertEqual(len(caps), 1)
        cap = caps[0]

        caption = "XXX Test DHCP Bond"
        rc = self.wbemconnection.InvokeMethod("LMI_CreateIPSetting", cap.path,
                Caption=caption,
                Type=pywbem.Uint16(4), # Bonding
                IPv4Type=pywbem.Uint16(4), # DHCP
                IPv6Type=pywbem.Uint16(9)) # Stateless

        self.assertEqual(len(rc), 2)
        self.assertEqual(rc[0], 0)
        self.assertTrue(rc[1] is not None)
        settingData = self.wbemconnection.GetInstance(rc[1]["SettingData"])

        slaves = self.wbemconnection.AssociatorNames(settingData.path,
                ResultClass="LMI_BondingSlaveSettingData",
                AssocClass="LMI_OrderedIPAssignmentComponent")
        self.assertEqual(len(slaves), 1)

        # Save the slave SettingData
        slaveSettingDataList = {}
        slaveSettingDataList[self.networkConnections[0]] = slaves[0]

        # Delete the connection on cleanup
        # Deleting master connection will also delete the slaves
        self.addCleanup(self.wbemconnection.DeleteInstance, settingData.path)

        # Check object presence
        for sd in self.wbemconnection.EnumerateInstances("LMI_BondingMasterSettingData"):
            if sd["Caption"] == caption:
                break
        else:
            self.assertTrue(False, "No instance of LMI_BondingMasterSettingData with Caption \"%s\"" % caption)

        # Check if properties are present
        self.assertTrue(settingData["InterfaceName"] is not None)
        self.assertEqual(settingData["Mode"], 0)
        self.assertEqual(settingData["MIIMon"], 100)
        self.assertEqual(settingData["DownDelay"], 0)
        self.assertEqual(settingData["UpDelay"], 0)
        self.assertEqual(settingData["ARPInterval"], 0)
        self.assertEqual(len(settingData["ARPIPTarget"]), 0)

        # Modify some options
        settingData["Mode"] = pywbem.Uint16(1)
        settingData["MIIMon"] = pywbem.Uint64(0)
        settingData["ARPInterval"] = pywbem.Uint64(1)
        settingData["ARPIPTarget"] = ["192.168.1.1", "192.168.1.2"]
        self.wbemconnection.ModifyInstance(settingData)
        settingData = self.wbemconnection.GetInstance(settingData.path)

        # Check if the options are modified
        self.assertEqual(settingData["Mode"], 1)
        self.assertEqual(settingData["ARPInterval"], 1)
        self.assertEqual(len(settingData["ARPIPTarget"]), 2)
        self.assertListEqual(settingData["ARPIPTarget"], ["192.168.1.1", "192.168.1.2"])

        # Modify some other options
        settingData["Mode"] = pywbem.Uint16(2)
        settingData["MIIMon"] = pywbem.Uint64(200)
        settingData["DownDelay"] = pywbem.Uint64(200)
        settingData["UpDelay"] = pywbem.Uint64(200)
        settingData["ARPInterval"] = pywbem.Uint64(0)
        settingData["ARPIPTarget"] = pywbem.CIMProperty("ARPIPTarget", [], type='string', is_array=True)
        self.wbemconnection.ModifyInstance(settingData)
        settingData = self.wbemconnection.GetInstance(settingData.path)

        # Check if the options are modified
        self.assertEqual(settingData["Mode"], 2)
        self.assertEqual(settingData["MIIMon"], 200)
        self.assertEqual(settingData["DownDelay"], 200)
        self.assertEqual(settingData["UpDelay"], 200)
        self.assertEqual(settingData["ARPInterval"], 0)
        self.assertEqual(len(settingData["ARPIPTarget"]), 0)

        # Add slaves, first one is added automatically when creating IP setting
        for networkConnection in self.networkConnections[1:]:
            caps = self.wbemconnection.Associators(networkConnection.path, ResultClass="LMI_IPNetworkConnectionCapabilities")
            self.assertEqual(len(caps), 1)
            cap = caps[0]

            rc = self.wbemconnection.InvokeMethod("LMI_CreateSlaveSetting", cap.path, MasterSettingData=settingData.path)
            self.assertEqual(len(rc), 2)
            self.assertEqual(rc[0], 0)
            self.assertEqual(rc[1]["SettingData"].classname, "LMI_BondingSlaveSettingData")
            slaveSettingDataList[networkConnection] = rc[1]["SettingData"]

        # Check if the slave SettingDatas are associated to master
        assoc = self.wbemconnection.Associators(settingData.path, ResultClass="LMI_BondingSlaveSettingData")
        self.assertEqual(len(assoc), len(self.networkConnections))

        # Test modification of slave setting data
        slaveSettingData = assoc[0]
        caption = "%s Modified" % slaveSettingData["Caption"]
        slaveSettingData["Caption"] = caption
        self.wbemconnection.ModifyInstance(slaveSettingData)
        slaveSettingData = self.wbemconnection.GetInstance(slaveSettingData.path)

        # Check if the properties are modified
        self.assertEqual(slaveSettingData["Caption"], caption)

        # Activate connection using SlaveSettingData
        for networkConnection in self.networkConnections:
            rc = self.wbemconnection.InvokeMethod("ApplySettingToIPNetworkConnection", self.confService.path,
                    SettingData=slaveSettingDataList[networkConnection], IPNetworkConnection=networkConnection.path, Mode=pywbem.Uint16(32768))
            self.assertIn(rc[0], [0, 4096])
            if rc[0] == 4096: # Job started
                # Wait for job completed indication
                self.wait_for_job_completion(rc[1]["Job"])

        # Check presence of master port
        aggregators = self.wbemconnection.Associators(sd.path, ResultClass="LMI_LinkAggregator8023ad",
                AssocClass="LMI_IPElementSettingData")
        self.assertGreater(len(aggregators), 0)
        aggregator = aggregators[0]

        lagPorts = self.wbemconnection.EnumerateInstances("LMI_LAGPort8023ad")

        # Check presence of LMI_LAGPort8023ad for each slave
        lagPortNames = [x["Name"] for x in lagPorts]
        for networkConnection in self.networkConnections:
            self.assertIn(networkConnection["Name"], lagPortNames)

        # Check if the slaves are associated with master
        aggregatedPorts = self.wbemconnection.Associators(aggregator.path, ResultClass="LMI_LAGPort8023ad",
                AssocClass="LMI_LinkAggregationBindsTo")
        for aggregatedPort in aggregatedPorts:
            self.assertIn(aggregatedPort["Name"], lagPortNames)

        # Check if slaves are aggregated to LANEndpoints
        for lagPort in lagPorts:
            lanendpoints = self.wbemconnection.Associators(lagPort.path, ResultClass="LMI_LANEndpoint",
                    AssocClass="LMI_LinkAggregationConcreteIdentity")
            self.assertEqual(len(lanendpoints), 1)
            self.assertEqual(lanendpoints[0]["Name"], lagPort["Name"])

    @classmethod
    def setUpClass(cls):
        TestBase.setUpClass()

    @classmethod
    def tearDownClass(cls):
        TestBase.tearDownClass()
        # Give it some time to actually delete the connections
        time.sleep(1)
        connections = []
        for connection in cls.wbemconnection.EnumerateInstances("LMI_IPAssignmentSettingData"):
            if connection["Caption"].startswith("XXX "):
                connections.append(connection["Caption"])
        if len(connections) > 0:
            raise AssertionError("Connections was not deleted: %s" % ", ".join(connections))



#!/usr/bin/python
# -*- Coding:utf-8 -*-
#
# Copyright (C) 2012 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Radek Novacek <rnovacek@redhat.com>
#

import os
import sys
import pywbem
import time

from test_base import TestBase

ENABLED = 2
DISABLED = 3

NM_UNAVAILABLE = 20
NM_DISCONNECTED = 30
NM_CONNECTED = 100

def waitForValues(values, function, *p):
    value = None
    for i in range(10):
        value = function(*p)
        if value in values:
            return value
        time.sleep(1)
    return value

class TestEnableDisableIface(TestBase):
    def setUp(self):
        TestBase.setUp(self)
        ports = self.wbemconnection.ExecQuery("WQL", 'select * from LMI_LANEndpoint where Name="%s"' % self.port)
        self.assertEqual(len(ports), 1, "Port with name %s doesn't exist, or is not managed by NetworkManager" % self.port)
        self.ethport = ports[0]
        # Create default connection, so we're able to activate the port
        self.defaultConnectionCaption = "Test Default %s" % self.port
        self.confService = self.wbemconnection.EnumerateInstances("LMI_IPConfigurationService")[0]

        ipnetcons = self.wbemconnection.Associators(self.ethport.path, ResultClass="LMI_IPNetworkConnection")
        self.assertEqual(len(ipnetcons), 1)
        ipnetcon = ipnetcons[0]

        capas = self.wbemconnection.Associators(ipnetcon.path, ResultClass="LMI_IPNetworkConnectionCapabilities")
        self.assertEqual(len(capas), 1)
        capa = capas[0]
        rc = self.wbemconnection.InvokeMethod("LMI_CreateIPSetting", capa.path,
                Caption=self.defaultConnectionCaption, IPv4Type=pywbem.Uint16(4)) # DHCP

    def tearDown(self):
        for instance in self.wbemconnection.ExecQuery("WQL", 'select * from LMI_IPAssignmentSettingData where Caption="%s"' % self.defaultConnectionCaption):
            self.wbemconnection.DeleteInstance(instance.path)

    def getState(self, port):
        port = self.wbemconnection.GetInstance(port.path)
        if port['EnabledState'] == 2:
            return NM_CONNECTED
        else:
            return NM_UNAVAILABLE

    def test_enableDisableIface(self):
        new_state = self.getState(self.ethport)
        for i in range(2):
            state = new_state
            if state in [NM_CONNECTED, NM_DISCONNECTED]:
                self.wbemconnection.InvokeMethod("RequestStateChange", self.ethport.path, RequestedState=pywbem.Uint16(DISABLED))
                new_state = waitForValues([NM_UNAVAILABLE], self.getState, self.ethport)
                self.assertEqual(new_state, NM_UNAVAILABLE, "Cannot disable device %s" % self.ethport["Name"])
            elif state == NM_UNAVAILABLE:
                self.wbemconnection.InvokeMethod("RequestStateChange", self.ethport.path, RequestedState=pywbem.Uint16(ENABLED))
                new_state = waitForValues([NM_CONNECTED, NM_DISCONNECTED], self.getState, self.ethport)
                self.assertIn(new_state, [NM_CONNECTED, NM_DISCONNECTED], "Cannot enable device %s" % self.ethport["Name"])




import sys
from test_base import TestBase, unittest
import pywbem
import time

class TestIndications(TestBase):
    #def setUp(self):
    #    TestBase.setUp(self)

    def test_connection_lifetime_indications(self):
        ipNetworkConnection = None
        for networkConnection in self.wbemconnection.EnumerateInstances("LMI_IPNetworkConnection"):
            if networkConnection["Name"] == self.port:
                ipNetworkConnection = networkConnection
        self.assertTrue(ipNetworkConnection is not None)
        capas = self.wbemconnection.Associators(ipNetworkConnection.path, ResultClass="LMI_IPNetworkConnectionCapabilities")
        self.assertGreater(len(capas), 0)
        capa = capas[0]

        # Connection creation indication
        filter_name = "test_create_%d" % (time.time() * 1000)
        sub = self.subscribe(filter_name, "select * from LMI_NetworkInstCreation where SourceInstance isa LMI_IPAssignmentSettingData")

        caption = "XXX Test Indication"
        rc = self.wbemconnection.InvokeMethod("LMI_CreateIPSetting", capa.path,
                Caption=caption, Type=pywbem.Uint16(1),
                IPv4Type=pywbem.Uint16(4))
        self.assertEqual(len(rc), 2)
        self.assertEqual(rc[0], 0)
        settingData = self.wbemconnection.GetInstance(rc[1]["SettingData"])

        indication = self.get_indication(10)
        self.assertEqual(indication.classname, "LMI_NetworkInstCreation")
        self.assertIn("SourceInstance", indication.keys())
        self.assertTrue(indication["SourceInstance"] is not None)
        self.assertEqual(indication["SourceInstance"]["Caption"], caption)

        # Connection modification indication
        filter_name = "test_modify_%d" % (time.time() * 1000)
        sub2 = self.subscribe(filter_name, "select * from LMI_NetworkInstModification where SourceInstance isa LMI_IPAssignmentSettingData")

        new_caption = caption + " Modified"
        settingData["Caption"] = new_caption
        self.wbemconnection.ModifyInstance(settingData)

        indication = self.get_indication(10)
        self.assertEqual(indication.classname, "LMI_NetworkInstModification")
        self.assertIn("SourceInstance", indication.keys())
        source = indication["SourceInstance"]
        self.assertEqual(source["Caption"], new_caption)

        self.assertIn("PreviousInstance", indication.keys())
        previous = indication["PreviousInstance"]
        self.assertEqual(previous["Caption"], caption)

        # Connection deletion indication
        filter_name = "test_delete_%d" % (time.time() * 1000)
        sub3 = self.subscribe(filter_name, "select * from LMI_NetworkInstDeletion where SourceInstance isa LMI_IPAssignmentSettingData")

        self.wbemconnection.DeleteInstance(settingData.path)

        indication = self.get_indication(10)
        self.assertEqual(indication.classname, "LMI_NetworkInstDeletion")
        self.assertIn("SourceInstance", indication.keys())
        source = indication["SourceInstance"]
        self.assertEqual(source["Caption"], new_caption)

    def test_port_lifetime_indications(self):
        # Create SettingData for testing
        ipNetworkConnection = None
        for networkConnection in self.wbemconnection.EnumerateInstances("LMI_IPNetworkConnection"):
            if networkConnection["Name"] == self.port:
                ipNetworkConnection = networkConnection
        self.assertTrue(ipNetworkConnection is not None)
        capas = self.wbemconnection.Associators(ipNetworkConnection.path, ResultClass="LMI_IPNetworkConnectionCapabilities")
        self.assertGreater(len(capas), 0)
        capa = capas[0]
        rc = self.wbemconnection.InvokeMethod("LMI_CreateIPSetting", capa.path,
                Caption="XXX Test Indication", Type=pywbem.Uint16(1),
                IPv4Type=pywbem.Uint16(4))
        self.assertEqual(len(rc), 2)
        self.assertEqual(rc[0], 0)
        settingData = rc[1]["SettingData"]

        services = self.wbemconnection.EnumerateInstances("LMI_IPConfigurationService")
        self.assertGreater(len(services), 0)
        ipConfService = services[0]

        # Subscribe
        filter_name = "test_port_modify_%d" % (time.time() * 1000)
        sub = self.subscribe(filter_name, "select * from LMI_NetworkInstModification where SourceInstance isa LMI_IPNetworkConnection")

        # Emit event
        rc = self.wbemconnection.InvokeMethod("ApplySettingToIPNetworkConnection", ipConfService.path,
                SettingData=settingData, IPNetworkConnection=ipNetworkConnection.path, Mode=pywbem.Uint32(32768))
        self.assertGreater(len(rc), 0)
        self.assertIn(rc[0], [0, 4096])

        while 1:
            indication = self.get_indication(10)
            if indication.classname == "LMI_NetworkInstModification":
                self.assertIn("SourceInstance", indication.keys())
                source = indication["SourceInstance"]
                break

    def tearDown(self):
        for settingData in self.wbemconnection.EnumerateInstances("LMI_IPAssignmentSettingData"):
            if settingData["Caption"] == "XXX Test Indication":
                self.wbemconnection.DeleteInstance(settingData.path)

if __name__ == '__main__':
    import os
    os.environ["LMI_CIMOM_URL"] = "local://pegasus"
    unittest.main()



#!/usr/bin/python
# -*- Coding:utf-8 -*-
#
# Copyright (C) 2012 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Radek Novacek <rnovacek@redhat.com>
#
# This test creates and destroys multiple connections.
# It tests creating connections using LMI_CreateIPSetting method of
# LMI_IPNetworkConnectionCapabilities

import sys
import os
import pywbem
import time
import IPy
import subprocess

import dbus
from dbus.mainloop.glib import DBusGMainLoop
import gobject

from test_base import TestBase

STATIC, DHCP = range(2)

# NetworkManager 0.8 uses different ObjectPath/Interface
nm_version = subprocess.Popen(['rpm', '-q', 'NetworkManager'], stdout=subprocess.PIPE).communicate()[0]
if nm_version.startswith('NetworkManager-0.8'):
    OBJECT_PATH = "/org/freedesktop/NetworkManagerSettings"
    IFACE_SETTINGS = "org.freedesktop.NetworkManagerSettings"
    IFACE_CONNECTION = "org.freedesktop.NetworkManagerSettings.Connection"
else:
    OBJECT_PATH = "/org/freedesktop/NetworkManager/Settings"
    IFACE_SETTINGS = "org.freedesktop.NetworkManager.Settings"
    IFACE_CONNECTION = "org.freedesktop.NetworkManager.Settings.Connection"


def intToIPv4(ip):
    return ".".join([str(x) for x in (ip % 256, (ip >> 8) % 256, (ip >> 16) % 256, (ip >> 24) % 256)])

def bytesToIPv6(ip):
    s = ""
    a = False
    for b in ip:
        s += "%.2x" % b
        if a:
            s += ":"
        a = not a
    return s[:-1]

def bytesToMac(mac):
    s = ""
    for b in mac:
        s += "%.2X:" % b
    return s[:-1]

class Connection(object):
    def __init__(self, caption, type_, ipv4=None, ipv6=None, port=None,
            ipv4_dns=None, ipv4_search=None, ipv6_dns=None, ipv6_search=None,
            ipv4_routes=None, ipv6_routes=None):

        # Mark our created connections with XXX for easier deletion
        self.caption = "XXX " + caption
        self.type = type_
        self.ipv4 = ipv4
        self.ipv6 = ipv6
        self.ipv4_dns = ipv4_dns
        self.ipv4_search = ipv4_search
        self.ipv6_dns = ipv6_dns
        self.ipv6_search = ipv6_search
        self.ipv4_routes = ipv4_routes
        self.ipv6_routes = ipv6_routes
        self.port = port

    def modifyInstance(self, instance):
        if instance.classname == "LMI_ExtendedStaticIPAssignmentSettingData":
            if instance["ProtocolIFType"] == 4096:
                ipaddresses = []
                subnetmasks = []
                gateways = []
                for addr in self.ipv4:
                    ipaddresses.append(addr[0])
                    subnetmasks.append(IPy.IP("0.0.0.0/%d" % addr[1]).netmask().strFullsize())
                    if len(addr) > 2:
                        gateways.append(addr[2])
                    else:
                        gateways.append("")
                instance['IPAddresses'] = ipaddresses
                instance['SubnetMasks'] = subnetmasks
                instance['GatewayAddresses'] = gateways
            elif instance["ProtocolIFType"] == 4097:
                ipaddresses = []
                prefixes = []
                gateways = []
                for addr in self.ipv6:
                    ipaddresses.append(addr[0])
                    prefixes.append(pywbem.Uint16(addr[1]))
                    if len(addr) > 2:
                        gateways.append(addr[2])
                    else:
                        gateways.append("")
                instance['IPAddresses'] = ipaddresses
                instance['IPv6SubnetPrefixLengths'] = prefixes
                instance['GatewayAddresses'] = gateways
        elif instance.classname == "LMI_DNSSettingData":
            if instance["ProtocolIFType"] == 4096:
                if self.ipv4_dns is not None:
                    instance["DNSServerAddresses"] = pywbem.CIMProperty(
                            "DNSServerAddresses",
                            self.ipv4_dns,
                            type='string',
                            array_size=len(self.ipv4_dns),
                            is_array=True)

                if self.ipv4_search is not None:
                    instance["DNSSearchDomains"] = pywbem.CIMProperty(
                            "DNSSearchDomains",
                            self.ipv4_search,
                            type='string',
                            array_size=len(self.ipv4_search),
                            is_array=True)
            elif instance["ProtocolIFType"] == 4097:
                if self.ipv6_dns is not None:
                    instance["DNSServerAddresses"] = pywbem.CIMProperty(
                            "DNSServerAddresses",
                            self.ipv6_dns,
                            type='string',
                            array_size=len(self.ipv6_dns),
                            is_array=True)
                if self.ipv6_search is not None:
                    instance["DNSSearchDomains"] = pywbem.CIMProperty(
                            "DNSSearchDomains",
                            self.ipv6_search,
                            type='string',
                            array_size=len(self.ipv6_search),
                            is_array=True)
        return instance

class TestCreatingConnections(TestBase):
    def runTest(self):
        self.test_CreateStaticConnection()
        self.test_CreateDHCPConnection()

    @classmethod
    def setUpClass(cls):
        TestBase.setUpClass()
        gobject.threads_init()
        mainloop = DBusGMainLoop()
        if cls.fake_test:
            cls.bus = dbus.SessionBus(mainloop=mainloop)
        else:
            cls.bus = dbus.SystemBus(mainloop=mainloop)

        cls.settings = cls.bus.get_object(cls.bus_name, OBJECT_PATH)

        cls.connections = []
        cls.ports = cls.wbemconnection.ExecQuery("WQL", 'select * from LMI_IPNetworkConnection where Name = "%s"' % cls.port)
        if len(cls.ports) > 0:
            cls.portref = cls.ports[0]

        # Register NewConnection signal
        cls.settings.connect_to_signal("NewConnection", cls.newConnection, dbus_interface=IFACE_SETTINGS)

        cls.confService = cls.wbemconnection.EnumerateInstances("LMI_IPConfigurationService")[0]

        cls.macs = {}
        for lan in cls.wbemconnection.EnumerateInstances("LMI_LANEndpoint"):
            cls.macs[lan["Name"]] = lan["MACAddress"]

    @classmethod
    def tearDownClass(cls):
        TestBase.tearDownClass()
        # Give it some time to delete the connections
        time.sleep(3)
        connections = []
        for connection in cls.wbemconnection.EnumerateInstances("LMI_IPAssignmentSettingData"):
            if connection["Caption"] is not None and connection["Caption"].startswith("XXX "):
                connections.append(connection["Caption"])
        if len(connections) > 0:
            raise AssertionError("Connections was not deleted: %s" % ", ".join(connections))

    def tearDown(self):
        TestBase.tearDown(self)
        try:
            self.wbemconnection.DeleteInstance(self.con.ref)
        except AttributeError:
            pass

    def setUp(self):
        TestBase.setUp(self)
        self.con = None
        self.assertGreater(len(self.ports), 0, "No network port found")
        # Subscribe to indication
        self.filter_name = "test_apply_%d" % (time.time() * 1000)
        self.subscribe(self.filter_name, "select * from LMI_NetworkInstModification where SourceInstance isa LMI_NetworkJob")

    def findDBusConnection(self, caption):
        for path in self.settings.ListConnections(dbus_interface=IFACE_SETTINGS):
            settings = self.bus.call_blocking(self.bus_name, path, IFACE_CONNECTION, "GetSettings", "", [])
            c = settings.get("connection", {}).get("id", None)
            if c is None:
                continue
            if c == caption:
                return settings

    def newConnection(self, path):
        print "newConnection", path

    def connectionCreate(self, connection):
        caps = self.wbemconnection.Associators(connection.port.path, ResultClass="LMI_IPNetworkConnectionCapabilities")
        self.assertEqual(len(caps), 1)
        cap = caps[0]

        if connection.type == STATIC:
            ipv4type = 3 # static
            ipv6type = 3 # static
        else:
            ipv4type = 4 # DHCP
            ipv6type = 9 # Stateless

        if not connection.ipv4:
            ipv4type = 0 # Disabled
        if not connection.ipv6:
            ipv6type = 0 # Disabled

        rc = self.wbemconnection.InvokeMethod("LMI_CreateIPSetting", cap.path,
                Caption=connection.caption,
                Type=pywbem.Uint16(1), # Ethernet
                IPv4Type=pywbem.Uint16(ipv4type),
                IPv6Type=pywbem.Uint16(ipv6type))

        self.assertEqual(rc[0], 0, "Creating connection has invalid return code: %d" % rc[0])
        try:
            connection.ref = rc[1]["SettingData"]
        except KeyError:
            self.assertTrue(False, "Method LMI_CreateIPSetting didn't return proper SettingData reference: %s" % rc[1])

        self.assertEqual(connection.ref.classname, "LMI_IPAssignmentSettingData")
        self.assertEqual(self.wbemconnection.GetInstance(connection.ref)["AddressOrigin"], 11) # Cumulative setting

        for assoc in self.wbemconnection.Associators(connection.ref, AssocClass="LMI_OrderedIPAssignmentComponent"):
            if assoc.classname == "LMI_DNSSettingData":
                continue
            if assoc["ProtocolIFType"] == 4096:
                self.assertEqual(assoc["AddressOrigin"], ipv4type)
                if ipv4type == 3:
                    self.assertEqual(assoc.classname, "LMI_ExtendedStaticIPAssignmentSettingData")
                elif ipv4type == 4:
                    self.assertEqual(assoc.classname, "LMI_DHCPSettingData")
            elif assoc["ProtocolIFType"] == 4097:
                self.assertEqual(assoc["AddressOrigin"], ipv6type)
                if ipv4type == 3:
                    self.assertEqual(assoc.classname, "LMI_ExtendedStaticIPAssignmentSettingData")
                elif ipv4type == 6:
                    self.assertEqual(assoc.classname, "LMI_DHCPSettingData")
                elif ipv4type == 9:
                    self.assertEqual(assoc.classname, "LMI_IPAssignmentSettingData")
            else:
                self.assertTrue(True, "Unknown ProtocolIFType: %s" % assoc["ProtocolIFType"])

        # Modify Static connections:
        for assoc in self.wbemconnection.Associators(connection.ref, AssocClass="LMI_OrderedIPAssignmentComponent"):
            if assoc.classname == "LMI_ExtendedStaticIPAssignmentSettingData":
                connection.modifyInstance(assoc)
                self.wbemconnection.ModifyInstance(assoc)
            if assoc.classname == "LMI_DNSSettingData":
                connection.modifyInstance(assoc)
                self.wbemconnection.ModifyInstance(assoc)

        routes = []
        if connection.ipv4_routes is not None:
            for route in connection.ipv4_routes:
                routes.append((pywbem.Uint16(1), # IPv4
                      route[0],
                      IPy.IP("0.0.0.0/%d" % route[1]).netmask().strFullsize(),
                      route[2],
                      pywbem.Uint16(route[3])))
        if connection.ipv6_routes is not None:
            for route in connection.ipv6_routes:
                routes.append((pywbem.Uint16(2), # IPv6
                      route[0],
                      pywbem.Uint8(route[1]),
                      route[2],
                      pywbem.Uint16(route[3])))

        for route in routes:
            args = {
                "AddressType": route[0],
                "DestinationAddress": route[1],
            }
            if route[0] == pywbem.Uint16(1):
                args["DestinationMask"] = route[2]
            else:
                args["PrefixLength"] = route[2]
            rc = self.wbemconnection.InvokeMethod("LMI_AddStaticIPRoute", connection.ref, **args)
            self.assertEqual(rc[0], 0)
            self.assertIn("Route", rc[1])
            routeSD = self.wbemconnection.GetInstance(rc[1]["Route"])
            routeSD["NextHop"] = route[3]
            routeSD["RouteMetric"] = route[4]
            rc = self.wbemconnection.ModifyInstance(routeSD)

    def connectionTest(self, connection):
        dbusConnection = self.findDBusConnection(connection.caption)
        if connection.type == STATIC:
            if connection.ipv4 is not None:
                self.assertIn("ipv4", dbusConnection)
                self.assertIn("method", dbusConnection["ipv4"])
                self.assertEqual(dbusConnection["ipv4"]["method"], "manual")
                self.assertIn("addresses", dbusConnection["ipv4"])
                addresses = dbusConnection["ipv4"]["addresses"]
                self.assertEqual(len(addresses), len(connection.ipv4))
                for i in range(len(addresses)):
                    self.assertEqual(connection.ipv4[i][0], intToIPv4(addresses[i][0]))
                    self.assertEqual(connection.ipv4[i][1], addresses[i][1])
                    if len(connection.ipv4[i]) > 2:
                        self.assertEqual(connection.ipv4[i][2], intToIPv4(addresses[i][2]))
                if connection.ipv4_dns is not None:
                    self.assertIn("dns", dbusConnection["ipv4"])
                    ip4stringlist = [intToIPv4(ip) for ip in dbusConnection["ipv4"]["dns"]]
                    self.assertListEqual(connection.ipv4_dns, ip4stringlist)

            if connection.ipv6 is not None:
                self.assertIn("ipv6", dbusConnection)
                self.assertIn("method", dbusConnection["ipv6"])
                self.assertEqual(dbusConnection["ipv6"]["method"], "manual")
                self.assertIn("addresses", dbusConnection["ipv6"])
                addresses = dbusConnection["ipv6"]["addresses"]
                self.assertEqual(len(addresses), len(connection.ipv6))
                for i in range(len(addresses)):
                    self.assertEqual(connection.ipv6[i][0], bytesToIPv6(addresses[i][0]))
                    self.assertEqual(connection.ipv6[i][1], addresses[i][1])
                    if len(connection.ipv6[i]) > 2:
                        self.assertEqual(connection.ipv6[i][2], bytesToIPv6(addresses[i][2]))
                if connection.ipv6_dns is not None:
                    self.assertIn("dns", dbusConnection["ipv6"])
                    ip6stringlist = [bytesToIPv6(ip) for ip in dbusConnection["ipv6"]["dns"]]
                    self.assertListEqual(connection.ipv6_dns, ip6stringlist)

            # dns-search property is IP-version agnostic
            dnssearch = set()
            if "ipv4" in dbusConnection and "dns-search" in dbusConnection["ipv4"]:
                for s in dbusConnection["ipv4"]["dns-search"]:
                    dnssearch.add(s)
            if "ipv6" in dbusConnection and "dns-search" in dbusConnection["ipv6"]:
                for s in dbusConnection["ipv6"]["dns-search"]:
                    dnssearch.add(s)
            if connection.ipv4_search is not None:
                for search in connection.ipv4_search:
                    self.assertIn(search, dnssearch)
            if connection.ipv6_search is not None:
                for search in connection.ipv6_search:
                    self.assertIn(search, dnssearch)

        else:
            if connection.ipv4:
                self.assertIn("ipv4", dbusConnection)
                self.assertIn("method", dbusConnection["ipv4"])
                self.assertEqual(dbusConnection["ipv4"]["method"], "auto")
            else:
                if "ipv4" in dbusConnection and "method" in dbusConnection["ipv4"]:
                    self.assertEqual(dbusConnection["ipv4"]["method"], "disabled")

            if connection.ipv6:
                self.assertIn("ipv6", dbusConnection)
                self.assertIn("method", dbusConnection["ipv6"])
                self.assertEqual(dbusConnection["ipv6"]["method"], "auto")
            else:
                if "ipv6" in dbusConnection and "method" in dbusConnection["ipv6"]:
                    self.assertEqual(dbusConnection["ipv6"]["method"], "ignore")

        if connection.ipv4_routes is not None:
            self.assertIn("ipv4", dbusConnection)
            self.assertIn("routes", dbusConnection["ipv4"])
            dbusRoutes = []
            for route in dbusConnection["ipv4"]["routes"]:
                dbusRoutes.append((intToIPv4(route[0]), route[1], intToIPv4(route[2]), route[3]))

            for route in connection.ipv4_routes:
                self.assertIn(route, dbusRoutes)

        if connection.ipv6_routes is not None:
            self.assertIn("ipv6", dbusConnection)
            self.assertIn("routes", dbusConnection["ipv6"])
            dbusRoutes = []
            for route in dbusConnection["ipv6"]["routes"]:
                dbusRoutes.append((bytesToIPv6(route[0]), route[1], bytesToIPv6(route[2]), route[3]))

            for route in connection.ipv6_routes:
                self.assertIn(route, dbusRoutes)

        if connection.port is not None:
            self.assertIn("802-3-ethernet", dbusConnection)
            self.assertIn("mac-address", dbusConnection["802-3-ethernet"])
            self.assertEqual(self.macs[connection.port["Name"]], bytesToMac(dbusConnection["802-3-ethernet"]["mac-address"]))

    def connectionDelete(self, connection):
        for i in self.wbemconnection.EnumerateInstances("LMI_IPAssignmentSettingData"):
            if i["Caption"] == connection.caption:
                self.wbemconnection.DeleteInstance(i.path)
        # wait for connection deletion
        time.sleep(2)

    def connectionApply(self, connection, mode=32768):
        rc = self.wbemconnection.InvokeMethod("ApplySettingToIPNetworkConnection", self.confService.path, SettingData=connection.ref, IPNetworkConnection=self.portref.path, Mode=pywbem.Uint16(mode))
        self.assertIn(rc[0], [0, 4096])
        if rc[0] == 4096:
            # Job is started, wait for finish
            job = self.wait_for_job(rc[1]["Job"], 60)
            self.assertEqual(job.classname, "LMI_NetworkJob")
            # Save the error message if the method fails
            errors = []
            if job["JobState"] != 7:
                for err in self.wbemconnection.InvokeMethod("GetErrors", rc[1]["Job"])[1]['Errors']:
                    errors.append(err["Message"])
            self.assertEqual(job["JobState"], 7, # Completed
                             "Unable to activate connection: %s" % ("; ".join(errors)))
    ## Static

    def test_static_both(self):
        """ Test creating and activating static connection with IPv4 and IPv6 """
        self.con = Connection("Test Static Both", STATIC,
                ipv4=[("192.168.122.201", 24, "192.168.147.100")],
                ipv6=[("2001:0db8:0000:2219:0210:18ff:fe97:2222", 24, "2001:0db8:0000:2219:0210:18ff:fe97:0001")],
                port=self.portref)
        self.connectionCreate(self.con)
        self.connectionTest(self.con)
        self.connectionApply(self.con)

    def test_static_both_dns(self):
        """ Test creating and activating static connection with IPv4 and IPv6 and dns records """
        # Set some DNS servers
        self.con = Connection("Test Static Both DNS", STATIC,
                ipv4=[("192.168.122.201", 24, "192.168.147.100")],
                ipv4_dns=["192.168.147.100", "192.168.147.101"],
                ipv4_search=["example.com", "test.example.com"],
                ipv6=[("2001:0db8:0000:2219:0210:18ff:fe97:2222", 24, "2001:0db8:0000:2219:0210:18ff:fe97:0001")],
                ipv6_dns=["2001:0db8:0000:2219:0210:18ff:fe97:0001", "2001:0db8:0000:2219:0210:18ff:fe97:0002"],
                ipv6_search=["example2.com", "test.example2.com"],
                port=self.portref)
        self.connectionCreate(self.con)
        self.connectionTest(self.con)
        self.connectionApply(self.con)

    def test_static_both_without_gateway(self):
        """ Test creating and activating static connection with IPv4 and IPv6 without default gateway """
        # Don't set gateway
        self.con = Connection("Test Static Both 2", STATIC,
                ipv4=[("192.168.147.1", 20)],
                ipv6=[("2001:0db8:0000:2219:0210:18ff:fe97:2222", 20)],
                port=self.portref)
        self.connectionCreate(self.con)
        self.connectionTest(self.con)
        self.connectionApply(self.con)

    def test_static_ipv4(self):
        """ Test creating and activating static connection with IPv4 only """
        self.con = Connection("Test Static IPv4", STATIC,
                ipv4=[("192.168.147.1", 24, "192.168.147.100")],
                ipv6=None,
                port=self.portref)
        self.connectionCreate(self.con)
        self.connectionTest(self.con)
        self.connectionApply(self.con)

    def test_static_ipv6(self):
        """ Test creating and activating static connection with IPv6 only """
        self.con = Connection("Test Static IPv6", STATIC,
                ipv4=None,
                ipv6=[("2001:0db8:0000:2219:0210:18ff:fe97:2222", 24, "2001:0db8:0000:2219:0210:18ff:fe97:0001")],
                port=self.portref)
        self.connectionCreate(self.con)
        self.connections.append(self.con)
        self.connectionTest(self.con)
        self.connectionApply(self.con)

    def test_static_fail(self):
        """ Test creating invalid static connection """
        self.con = Connection("Test Static None", STATIC,
                ipv4=None,
                ipv6=None,
                port=self.portref)
        self.assertRaises(pywbem.CIMError, self.connectionCreate, self.con)

    ## DHCP

    def test_dhcp_both(self):
        """ Test creating and activating DHCP connection with IPv4 and IPv6 """
        self.con = Connection("Test DHCP Both", DHCP,
                ipv4=True,
                ipv6=True,
                port=self.portref)
        self.connectionCreate(self.con)
        self.connectionTest(self.con)
        self.connectionApply(self.con)

    def test_dhcp_ipv4(self):
        """ Test creating and activating DHCP connection with IPv4 only """
        self.con = Connection("Test DHCP IPv4", DHCP,
                ipv4=True,
                ipv6=False,
                port=self.portref)
        self.connectionCreate(self.con)
        self.connectionTest(self.con)
        self.connectionApply(self.con)

    def test_dhcp_ipv6(self):
        """ Test creating and activating DHCP connection with IPv6 only """
        self.con = Connection("Test DHCP IPv6", DHCP,
                ipv4=False,
                ipv6=True,
                port=self.portref)
        self.connectionCreate(self.con)
        self.connectionTest(self.con)
        self.skipTest("This test requires DHCPv6 server, skipped")
        # self.connectionApply(self.con)

    def test_dhcp_fail(self):
        """ Test creating invalid DHCP connection """
        self.con = Connection("Test DHCP None", DHCP,
                ipv4=False,
                ipv6=False,
                port=self.portref)
        self.assertRaises(pywbem.CIMError, self.connectionCreate, self.con)

    def test_static_route(self):
        """ Test creating and activating static connection with routes"""
        self.con = Connection("Test Static Route", STATIC,
                ipv4=[("192.168.122.201", 24, "192.168.122.100")],
                ipv6=[("2001:0db8:0000:2219:0210:18ff:fe97:2222", 64, "2001:0db8:0000:2219:0210:18ff:fe97:0001")],
                ipv4_routes=[("192.168.123.0", 24, "192.168.122.101", 10),
                             ("192.168.124.0", 24, "192.168.122.102", 20)],
                ipv6_routes=[("2001:0db8:0000:2220:0000:0000:0000:0000", 64, "2001:0db8:0000:2219:0210:18ff:fe97:0002", 10),
                             ("2001:0db8:0000:2221:0000:0000:0000:0000", 64, "2001:0db8:0000:2219:0210:18ff:fe97:0003", 20)],
                port=self.portref)
        self.connectionCreate(self.con)
        self.connectionTest(self.con)
        self.connectionApply(self.con)



#!/usr/bin/python
# -*- Coding:utf-8 -*-

import sys
import os
import signal
import tempfile
import shutil
import subprocess
import pexpect
import threading

class IOThread(threading.Thread):
    def __init__(self, command, args=None, events=None, **kw):
        threading.Thread.__init__(self)
        self.kw = kw
        self.command = command
        if args is None:
            self.args = []
        else:
            self.args = args
        self.events = {}
        if events is not None:
            for ev in events:
                self.events[ev] = threading.Event()

        self.is_timeout = False
        self.end_event = threading.Event()

    def event(self, event):
        return self.events[event]

    def run(self, events=None):
        self.p = pexpect.spawn(self.command, self.args, **self.kw)
        while True:
            try:
                index = self.p.expect(self.events.keys() + ["\n"])
            except pexpect.EOF, e:
                self.end_event.set()
                break
            except pexpect.TIMEOUT, e:
                self.is_timeout = True
                self.end_event.set()
                break

            s = self.p.before
            if isinstance(self.p.after, basestring):
                s += self.p.after
            sys.stderr.write(s)
            if index < len(self.events):
                event = self.events[self.events.keys()[index]]
                event.before = s
                event.set()

    def kill(self):
        self.p.terminate(force=True)

class CIMOM(object):
    startString = "localConnectServer started"
    providerWaitString = "-#- Pausing for provider:.*"
    def __init__(self, mof_file, reg_file, provider_dir, debug=False):
        self.debug = debug
        self.temp_dir = None
        self._create_config(mof_file, reg_file, provider_dir)
        self._register()

        print "------ sfcbd"
        env = os.environ
        if self.debug:
            env["SFCB_PAUSE_PROVIDER"] = "LMI_IPConfigurationService"

        self.sfcb = IOThread("/usr/sbin/sfcbd", ["-c", self.config_file], env=env, events=[self.startString, self.providerWaitString])
        self.sfcb.start()
        self.sfcb.event(self.startString).wait()

    def __del__(self):
        if self.temp_dir is not None:
            if os.path.isdir(self.temp_dir):
                shutil.rmtree(self.temp_dir)

    def _create_config(self, mof_file, reg_file, provider_dir):
        self.mof_file = mof_file
        self.reg_file = reg_file
        self.provider_dir = provider_dir
        self.p = None

        self.temp_dir = tempfile.mkdtemp()
        self.config_file = os.path.join(self.temp_dir, "sfcb.conf")
        f = open(self.config_file, "w")
        f.write("""httpPort: 54320
providerDirs: /usr/lib64/sfcb/ %(provider_dir)s
registrationDir: %(temp_dir)s/registration
httpSocketPath: %(temp_dir)s/openlmi-sfcbHttpSocket""" % {
    'provider_dir': os.path.abspath(self.provider_dir),
    'temp_dir': self.temp_dir
})
        f.close()

    def _register(self):
        print "Starting sfcbd ..."
        staging_dir = os.path.join(self.temp_dir, "stage")
        os.mkdir(staging_dir)
        registration_dir = os.path.join(self.temp_dir, "registration")
        os.mkdir(registration_dir)

        shutil.copy("/var/lib/sfcb/stage/default.reg", staging_dir)
        print "------ sfcbstage"
        subprocess.call(["/usr/bin/sfcbstage", "-s", staging_dir, "-r", self.reg_file, "/usr/share/openlmi-providers/05_LMI_Qualifiers.mof", self.mof_file])

        print "------ sfcbrepos"
        subprocess.call(["/usr/bin/sfcbrepos", "-s", staging_dir, "-r", registration_dir, "-f"])

    def run_debugger(self):
        if self.debug:
            event = self.sfcb.event(self.providerWaitString)
            event.wait()
            line = event.before
            self.provider_pid = int(line[line.find("-pid: ") + 6:])
            print "Provider PID:", self.provider_pid

            self.gdb = pexpect.spawn("/usr/bin/gdb",
                ["-q", "-p", "%s" % self.provider_pid,
                 "-ex", "up 2", "-ex", "set debug_break=1",
                 "-ex", "set follow-fork-mode child"])
            threading.Thread(target=self.gdb.interact).start()

    def poll(self):
        return self.sfcb.isalive()

    def terminate(self):
        os.kill(self.sfcb.p.pid, signal.SIGTERM)



#!/usr/bin/python
# -*- Coding:utf-8 -*-
#
# Copyright (C) 2012 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Radek Novacek <rnovacek@redhat.com>
#
# Usage: ./test.py https://localhost:5989 root <password>
#
# Runs all defined tests

import sys
import os
import signal
import time

from optparse import OptionParser

import pexpect
import subprocess

from fake_cimom import CIMOM, IOThread

class LocalDBus(subprocess.Popen):
    def __init__(self):
        print "Starting dbus daemon"
        # Start private DBus daemon
        out = subprocess.check_output(['dbus-launch'], universal_newlines=True)
        variables = {}
        for line in out.splitlines():
            (k, v) = line.split('=', 1)
            variables[k] = v
        self.buspid = int(variables['DBUS_SESSION_BUS_PID'])
        os.environ["DBUS_SESSION_BUS_ADDRESS"] = variables['DBUS_SESSION_BUS_ADDRESS']

        testdir = os.path.dirname(__file__)

        subprocess.Popen.__init__(self, ["/usr/bin/python", os.path.join(testdir, "fake_nm.py")])

    def terminate(self):
        subprocess.Popen.terminate(self)
        try:
            os.environ.pop("DBUS_SESSION_BUS_ADDRESS")
        except KeyError:
            pass
        os.kill(self.buspid, signal.SIGTERM)

if __name__ == '__main__':
    parser = OptionParser(usage="usage: %prog [options]")
    parser.add_option("-f", "--fake", dest="fake", action="store_true", default=False,
                      help="Use fake NetworkManager API to aviod spoiling targeted system")
    parser.add_option("-p", "--ports", dest="ports", default=None, help="Network ports to use for tests (comma-separated), some tests require to use at least 2 ports")
    parser.add_option("-t", "--test", dest="tests", action="append")
    parser.add_option("-l", "--local", dest="path", help="Use local (not registered) build, with given path to provider library")
    parser.add_option("-d", "--debug", dest="debug", action="store_true", default=False, help="Use gdb to debug build")

    parser.add_option("--hostname", action="store", dest="hostname",
            default="https://localhost:5989",
            help="remote machine hostname")
    parser.add_option("--username", action="store", dest="username",
            default="pegasus",
            help="remote machine username")
    parser.add_option("--password", action="store", dest="password",
            default="pegasus",
            help="remote machine password")

    (options, args) = parser.parse_args()

    if options.fake and options.path is None:
        parser.error("Test can't be run in fake mode without local build")

    if options.path is not None:
        os.environ["LMI_CIMOM_URL"] = "http://localhost:54320"
        os.environ["LMI_CIMOM_USERNAME"] = ""
        os.environ["LMI_CIMOM_PASSWORD"] = ""
    else:
        os.environ["LMI_CIMOM_URL"] = options.hostname
        os.environ["LMI_CIMOM_USERNAME"] = options.username
        os.environ["LMI_CIMOM_PASSWORD"] = options.password

    if options.ports is None:
        if options.fake:
            # always use eth1 for local tests
            options.ports = "eth1,eth2"
        else:
            options.ports = ",".join(subprocess.check_output(["nmcli", "-f", "DEVICE", "-t", "dev"]).splitlines())
            if len(options.ports) < 1:
                sys.exit("No usable network port found (check result of command 'nmcli dev')")

    os.environ["LMI_NETWORKING_PORTS"] = options.ports
    if options.fake:
        os.environ["LMI_NETWORKING_BUS_NAME"] = "org.freedesktop.FakeNetworkManager"
        os.environ["LMI_NETWORKING_FAKE_NM"] = "1"
    else:
        os.environ["LMI_NETWORKING_FAKE_NM"] = "0"

    fake_nm = None
    fake_cimom = None

    if options.fake:
        fake_nm = LocalDBus()

    if options.path is not None:
        fake_cimom = CIMOM(
                os.path.join(os.path.dirname(os.path.dirname(os.path.abspath(__file__))), "mof", "LMI_Networking.mof"),
                os.path.join(options.path, "LMI_Networking.reg"), options.path,
                debug=options.debug)

    if options.tests is None:
        options.tests = []

    print "------ nosetests"
    nose = IOThread("/usr/bin/nosetests", ["-v", "--process-timeout=20"] + options.tests, env=os.environ, cwd=os.path.dirname(__file__), timeout=60)
    nose.start()
    if fake_cimom is not None:
        fake_cimom.run_debugger()

    nose.end_event.wait()
    if nose.is_timeout:
        print "------ nosetests TIMEOUT"
    else:
        print "------ nosetests done"

    if fake_nm is not None:
        fake_nm.terminate()
        fake_nm.wait()
    if fake_cimom is not None:
        fake_cimom.terminate()

    sys.exit(nose.p.exitstatus)



#!/usr/bin/python
# -*- Coding:utf-8 -*-
#
# Copyright (C) 2012 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Radek Novacek <rnovacek@redhat.com>
#
# Fake NetworkManager-like API for testing networking provider
#

import sys
import dbus
import dbus.service
import gobject

from dbus.mainloop.glib import DBusGMainLoop

def debug(s):
    sys.stderr.write("-fake- %s\r\n" % s.replace("\n", "\r\n-fake- "))

class Device(dbus.service.Object):
    index = 1
    def __init__(self, bus_name, name=None, type=None):
        self.index = Device.index
        Device.index += 1
        self.object_path = "/org/freedesktop/NetworkManager/Devices/%d" % self.index
        dbus.service.Object.__init__(self, bus_name, self.object_path)
        if name is None:
            self.name = 'eth%d' % self.index
        else:
            self.name = name
        self.type = type if type is not None else 1
        self.properties = {
                'State': dbus.UInt32(100),
                'Interface': self.name,
                'IpInterface': self.name,
                'DeviceType': dbus.UInt32(self.type),
                'Ip4Config': dbus.ObjectPath("/"),
                'Ip6Config': dbus.ObjectPath("/"),
                'Dhcp4Config': dbus.ObjectPath("/"),
                'Dhcp6Config': dbus.ObjectPath("/")
        }
        self.subProperties = dbus.Dictionary({
                'HwAddress': "AA:BB:CC:DD:EE:%02X" % self.index
            }, signature="sv")
        if self.type == 10:
            # Bonding
            self.subProperties["Slaves"] = dbus.Array(signature="o")

    @dbus.service.signal(dbus_interface="org.freedesktop.NetworkManager.Device.Bond",
                         signature="a{sv}")
    def PropertiesChanged(self, props):
        debug("Device.PropertiesChanged %s" % props)

    @dbus.service.method(dbus_interface="org.freedesktop.DBus.Properties",
                         in_signature="s",
                         out_signature="a{sv}")
    def GetAll(self, interface):
        debug("Device.GetAll")
        if interface == "org.freedesktop.NetworkManager.Device":
            return self.properties
        else:
            return self.subProperties

    @dbus.service.method(dbus_interface="org.freedesktop.DBus.Properties",
                         in_signature="ss",
                         out_signature="v")
    def Get(self, interface, prop):
        if interface == "org.freedesktop.NetworkManager.Device":
            return self.properties[prop]
        else:
            return self.subProperties[prop]


class ActiveConnection(dbus.service.Object):
    def __init__(self, bus_name, index, device, connection):
        self.object_path = "/org/freedesktop/NetworkManager/ActiveConnection/%d" % index
        dbus.service.Object.__init__(self, bus_name, self.object_path)
        self.index = index
        self.properties = {
            'Devices': [device.object_path],
            'Connection': connection.object_path,
            'State': 2
        }

    @dbus.service.method(dbus_interface="org.freedesktop.DBus.Properties",
                         in_signature="s",
                         out_signature="a{sv}")
    def GetAll(self, interface):
        debug("GetAll")
        return self.properties

    @dbus.service.method(dbus_interface="org.freedesktop.DBus.Properties",
                         in_signature="ss",
                         out_signature="v")
    def Get(self, interface, prop):
        debug("Get")
        if interface == 'org.freedesktop.NetworkManager.Connection.Active':
            if prop == 'State':
                return dbus.UInt32(self.properties[prop])



class Manager(dbus.service.Object):
    def __init__(self, bus_name):
        self.bus_name = bus_name
        self.object_path = "/org/freedesktop/NetworkManager"
        dbus.service.Object.__init__(self, bus_name, self.object_path)

        self.devices = {}
        self.activeConnections = {}
        self.connections = {}

        self.activeConnectionsIndex = 0
        self.connectionsIndex = 0

        self.addEthDevice()
        self.addEthDevice()

    def addEthDevice(self):
        d = Device(self.bus_name)
        self.addDevice(d)
        return d

    def addDevice(self, d):
        self.devices[d.object_path] = d
        self.DeviceAdded(d.object_path) # emit signal

    def connectionByUuid(self, uuid):
        for con in self.connections.values():
            if con.hash.get("connection", []).get("uuid", "") == uuid:
                return con
        return None

    def deviceByName(self, name):
        for dev in self.devices.values():
            if dev.name == name:
                return dev
        return None

    def deviceByMAC(self, mac):
        for dev in self.devices.values():
            if dev.subProperties.get("HwAddress") == mac:
                return dev
        print "No device with mac address:", mac
        return None


    @dbus.service.method(dbus_interface="org.freedesktop.NetworkManager",
                         in_signature="ooo",
                         out_signature="o")
    def ActivateConnection(self, connection, device, specific_object):
        debug("ActivateConnection")
        ac = ActiveConnection(self.bus_name, self.activeConnectionsIndex, self.devices[device], self.connections[connection])
        self.activeConnections[ac.object_path] = ac
        self.activeConnectionsIndex += 1
        return ac.object_path

    @dbus.service.method(dbus_interface="org.freedesktop.NetworkManager",
                         in_signature="",
                         out_signature="o")
    def DeactivateConnection(self):
        debug("DeactivateConnection")
        return ""

    @dbus.service.method(dbus_interface="org.freedesktop.NetworkManager",
                         in_signature="a{sa{sv}}oo",
                         out_signature="oo")
    def AddAndActivateConnection(self, connection, device, specific_object):
        debug("AddAndActivateConnection")
        return ("", "")

    @dbus.service.method(dbus_interface="org.freedesktop.NetworkManager",
                         in_signature="",
                         out_signature="ao")
    def GetDevices(self):
        debug("GetDevices")
        return [device.object_path for device in self.devices.values()]

    @dbus.service.method(dbus_interface="org.freedesktop.DBus.Properties",
                         in_signature="s",
                         out_signature="a{sv}")
    def GetAll(self, interface):
        debug("GetAll")
        return { "ActiveConnections": dbus.Array([], signature="o") }

    @dbus.service.signal(dbus_interface="org.freedesktop.NetworkManager",
                         signature="o")
    def DeviceRemoved(self, device):
        debug("DeviceRemoved %s" % device)

    @dbus.service.signal(dbus_interface="org.freedesktop.NetworkManager",
                         signature="o")
    def DeviceAdded(self, device):
        debug("DeviceAdded %s" % device)

    @dbus.service.signal(dbus_interface="org.freedesktop.NetworkManager",
                         signature="a{sv}")
    def PropertiesChanged(self, props):
        debug("PropertiesChanged %s" % props)

    @dbus.service.signal(dbus_interface="org.freedesktop.NetworkManager",
                         signature="u")
    def StateChanged(self, state):
        debug("StateChanged %s" % state)

class Settings(dbus.service.Object):
    def __init__(self, bus_name, manager):
        self.bus_name = bus_name
        self.manager = manager
        dbus.service.Object.__init__(self, bus_name, "/org/freedesktop/NetworkManager/Settings")
        self.settings = self.manager.connections
        self.settingsIndex = 0

    @dbus.service.method(dbus_interface="org.freedesktop.NetworkManager.Settings",
                         in_signature="a{sa{sv}}",
                         out_signature="o")
    def AddConnection(self, connection):
        c = Connection(self.manager, self.bus_name, self.settingsIndex, connection, self)
        self.settingsIndex += 1
        self.settings[c.object_path] = c
        con = connection.get("connection", [])
        if con.get("type", "") == "bond":
            debug("Bonding")
            # it's bonding, we need to add bond device
            d = Device(self.bus_name, connection["bond"]["interface-name"], 10)
            self.manager.addDevice(d)
        if "master" in con.keys():
            master_setting = self.manager.connectionByUuid(con["master"])
            master_name = master_setting.hash.get("bond", []).get("interface-name", "")
            device = self.manager.deviceByName(master_name)
            mac = ":".join(["%.2X" % x for x in connection.get("802-3-ethernet").get("mac-address")])
            device.subProperties["Slaves"].append(dbus.ObjectPath(self.manager.deviceByMAC(mac).object_path))
            device.PropertiesChanged(device.subProperties)
        self.NewConnection(c.object_path)
        return c.object_path

    @dbus.service.method(dbus_interface="org.freedesktop.NetworkManager.Settings",
                         in_signature="",
                         out_signature="ao")
    def ListConnections(self):
        return self.settings.keys()

    @dbus.service.signal(dbus_interface="org.freedesktop.NetworkManager.Settings",
                         signature="o")
    def NewConnection(self, objectpath):
        debug("NewConnection %s" % objectpath)


class Connection(dbus.service.Object):
    def __init__(self, manager, bus_name, index, hash, settings):
        self.manager = manager
        self.object_path = "/org/freedesktop/NetworkManager/Settings/%d" % index
        self.hash = hash
        self.settings = settings
        dbus.service.Object.__init__(self, bus_name, self.object_path)

    @dbus.service.method(dbus_interface="org.freedesktop.NetworkManager.Settings.Connection",
                         in_signature="",
                         out_signature="a{sa{sv}}")
    def GetSettings(self):
        return self.hash

    @dbus.service.method(dbus_interface="org.freedesktop.NetworkManager.Settings.Connection",
                         in_signature="",
                         out_signature="")
    def Delete(self):
        if self.hash["connection"]["type"] == "bond":
            iface = self.hash["bond"]["interface-name"]
            dev = self.manager.deviceByName(iface)
            self.manager.DeviceRemoved(dev.object_path)
            self.manager.devices.pop(dev.object_path)
        self.settings.settings.pop(self.object_path)
        self.Removed()

    @dbus.service.signal(dbus_interface="org.freedesktop.NetworkManager.Settings.Connection",
                         signature="")
    def Removed(self):
        debug("Removed %s" % self.object_path)

class Controller(dbus.service.Object):
    def __init__(self, bus_name, manager):
        self.bus_name = bus_name
        self.manager = manager
        dbus.service.Object.__init__(self, bus_name, "/org/fedoraproject/OpenLMINetworking/Controller")

    @dbus.service.method(dbus_interface="org.fedoraproject.OpenLMINetworking.Controller",
                         in_signature="s",
                         out_signature="")
    def AddDevice(self, name):
        self.manager.addEthDevice()

class FakeNM(object):
    loop = None
    doStart = True
    def __init__(self):
        mainloop = DBusGMainLoop()

        self.bus = dbus.SessionBus(mainloop=mainloop)

        bus_name = dbus.service.BusName('org.freedesktop.FakeNetworkManager', self.bus)
        manager = Manager(bus_name)
        setting = Settings(bus_name, manager)
        controller = Controller(bus_name, manager)

        self.loop = gobject.MainLoop()
        self.loop.run()

if __name__ == "__main__":
    gobject.threads_init()
    f = FakeNM()



#!/usr/bin/python
# -*- Coding:utf-8 -*-
#
# Copyright (C) 2012 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Radek Novacek <rnovacek@redhat.com>
#
# This test checks if all associations defined in objects.json are present
#

import sys
import os
import pywbem
import subprocess
import json
import pprint

from test_base import TestBase

wbemconnection = None

def readObjects(fileName):
    f = open(fileName, "r")
    return json.load(f)

def run_and_read(command):
    p = subprocess.Popen(command, stdout=subprocess.PIPE, shell=True)
    if p.wait() != 0:
        return None
    return p.communicate()[0]

def read(d, key):
    v = d.get(key, None)
    if v is not None:
        if v.startswith("%"):
            return wbemconnection.ExecQuery("WQL", v[1:])
        else:
            return v
    return None

def readList(d, key):
    l = read(d, key)
    if type(l) in [str, unicode]:
        return l.strip().split("\n")
    else:
        return l

def readInt(d, key):
    try:
        i = read(d, key)
    except pywbem.CIMError:
        return 0
    if i is None:
        return None
    if type(i) == list:
        return len(i)
    else:
        return int(i)

def dict2str(d, indent=4, level=0):
    s = ""
    i = (" " * indent) * level
    t = type(d)
    if t == str or t == unicode:
        s += '"%s"' % d
    elif t == list:
        i2 = i + (" " * indent)
        s += "[\n"
        for item in d:
            s += i2 + dict2str(item, level=level+1) + ",\n"
        s += i + "]"
    elif t == dict:
        i2 = i + (" " * indent)
        s += "{\n"
        for key, value in d.items():
            s += i2 + dict2str(key, level=level+1) + ": " + dict2str(value, level=level+1) + ",\n"
        s += i + "}"
    else:
        s += str(d)
    return s

class TestObjects(TestBase):
    def setUp(self):
        TestBase.setUp(self)
        global wbemconnection
        wbemconnection = self.wbemconnection
        fileName = "objects.json"

        exeDir = os.path.dirname(sys.argv[0])
        for path in [os.curdir, os.path.join(os.curdir, "test"), exeDir, os.path.join(exeDir, "test")]:
            objectsPath = os.path.join(path, fileName)
            if os.path.exists(objectsPath):
                break
        else:
            self.fail("Unable to find file %s" % fileName)
        try:
            self.assoc = readObjects(objectsPath)
        except ValueError, e:
            self.fail("Unable to parse %s: %s" % (objectsPath, str(e)))

    def test_classes(self):
        for cls in self.assoc["classes"]:
            try:
                instances = self.wbemconnection.ExecQuery("WQL", "select * from %s" % cls["name"])
            except pywbem.CIMError, e:
                self.fail("Error querying the class %s: %s" % (cls["name"], e[1]))

            unusedValues = {}
            values = cls.get("values", [])
            for item in values:
                for key in item.keys():
                    val = readList(item, key)
                    unusedValues[key] = val[:]

            for instance in instances:
                for key, unusedValuesItem in unusedValues.items():
                    self.assertTrue(key in instance, "Class %s don't have key %s, definition: \n%s" % (cls["name"], key, dict2str(cls)))
                    try:
                        value = instance[key]
                    except KeyError:
                        self.fail("Class %s don't have required key %s, definition: \n%s" % (cls["name"], key, dict2str(cls)))

                    try:
                        unusedValuesItem.remove(value)
                    except ValueError:
                        self.fail("Class %s has key %s with value %s, expected one of %s, definition: \n%s" % (cls["name"], key, value, str(unusedValuesItem), dict2str(cls)))

            classCount = readInt(cls, "count")
            if classCount is not None:
                self.assertEqual(len(instances), classCount, "Class %s has wrong number of instances: %d, expected %d, definition: \n%s" % (cls["name"], len(instances), classCount, dict2str(cls)))

    def test_associations(self):
        for assoc in self.assoc["associations"]:
            try:
                instances = self.wbemconnection.ExecQuery("WQL", "select * from %s" % assoc["name"])
            except pywbem.CIMError, e:
                self.fail("Unable to query for class %s: %s" % (assoc["name"], str(e)))

            instanceCount = 0
            for instance in instances:
                fromClass = instance[assoc["from_role"]].classname
                toClass = instance[assoc["to_role"]].classname
                filtr = assoc.get("filter", None)
                if filtr is not None:
                    if "from_class" in filtr and fromClass != filtr["from_class"]:
                        continue
                    if "to_class" in filtr and toClass != filtr["to_class"]:
                        continue
                self.assertEqual(fromClass, assoc["from"], "Class %s should be associated to %s instead of %s via role %s, definition: \n%s" % (assoc["name"], assoc["from"], fromClass, assoc["from_role"], dict2str(assoc)))
                self.assertEqual(toClass, assoc["to"], "Class %s should be associated to %s instead of %s via role %s, definition: \n%s" % (assoc["name"], assoc["to"], toClass, assoc["to_role"], dict2str(assoc)))
                instanceCount += 1

            # Check the count
            assocCount = readInt(assoc, "count")
            if assocCount is not None:
                self.assertEqual(instanceCount, assocCount, "Class %s has wrong number of instances: %d, expected %d, definition: \n%s" % (assoc["name"], instanceCount, assocCount, dict2str(assoc)))




#!/usr/bin/python
# -*- Coding:utf-8 -*-
#
# Copyright (C) 2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Radek Novacek <rnovacek@redhat.com>
#
# This test tests changing connections.
# It tests modification of IPAssociationSettingData objects

import sys
import os
import pywbem
import time
import IPy

from test_base import TestBase, unittest


class TestModifyConnections(TestBase):
    @classmethod
    def setUpClass(cls):
        TestBase.setUpClass()

        cls.portrefs = []
        for port in cls.wbemconnection.EnumerateInstances("LMI_IPNetworkConnection"):
            if port["Name"] in cls.ports:
                cls.portrefs.append(port)
        if len(cls.portrefs) > 0:
            cls.portref = cls.portrefs[0]
        else:
            raise AssertionError("No network port found")

        cls.confService = cls.wbemconnection.EnumerateInstances("LMI_IPConfigurationService")[0]

        capas = cls.wbemconnection.Associators(cls.portref.path, ResultClass="LMI_IPNetworkConnectionCapabilities")
        if len(capas) != 1:
            raise AssertionError("No LMI_IPNetworkConnectionCapablities for port %s" % cls.portref["Name"])
        capa = capas[0]

        # Create connection for testing
        cls.caption = "XXX Test Connection"
        rc = cls.wbemconnection.InvokeMethod("LMI_CreateIPSetting",
            capa.path, Caption=cls.caption, IPv4Type=pywbem.Uint16(4)) # DHCP
        if rc[0] != 0:
            raise AssertionError("Unable to create connection")
        cls.settingData = rc[1]["SettingData"]

    @classmethod
    def tearDownClass(cls):
        TestBase.tearDownClass()
        for connection in cls.wbemconnection.EnumerateInstances("LMI_IPAssignmentSettingData"):
            if connection['AddressOrigin'] == 11 and connection["Caption"].startswith("XXX "):
                cls.wbemconnection.DeleteInstance(connection.path)

    def test_modify_caption(self):
        """ Test modify caption of the connection """
        self.caption += " Modified"
        self.settingData = self.wbemconnection.GetInstance(self.settingData)
        self.settingData["Caption"] = self.caption
        self.wbemconnection.ModifyInstance(self.settingData)

        for i in self.wbemconnection.EnumerateInstances("LMI_IPAssignmentSettingData"):
            if i["Caption"] == self.caption:
                break
        else:
            self.assertTrue(True, "Updating of Caption failed")

    @unittest.skip("Changing type of connection is not currently supported")
    def test_modify_ip_version(self):
        """ Test modify IP version of connection """
        sd = self.wbemconnection.GetInstance(self.settingData)
        self.assertEqual(sd["ProtocolIFType"], 4096) # IPv4

        rc = self.wbemconnection.InvokeMethod("CreateOrModifyDynamicSetting",
                self.confService.path, ProtocolIFType=pywbem.Uint16(2),
                SettingData=self.settingData)
        self.assertEqual(rc[0], 0)
        sd = self.wbemconnection.GetInstance(rc[1]["SettingData"])
        self.assertEqual(sd["ProtocolIFType"], 4097) # IPv6
        self.settingData = rc[1]["SettingData"]

    def test_modify_setting_method(self):
        """ Test modify setting method """

        # Set IPv4Type to Static
        self.settingData = self.wbemconnection.GetInstance(self.settingData)
        self.settingData["IPv4Type"] = pywbem.Uint16(3) # Static
        self.wbemconnection.ModifyInstance(self.settingData)
        # Check the changed value
        self.settingData = self.wbemconnection.GetInstance(self.settingData.path)
        self.assertEqual(self.settingData["IPv4Type"], pywbem.Uint16(3))
        # Check association
        assoc = self.wbemconnection.Associators(self.settingData.path, AssocClass="LMI_OrderedIPAssignmentComponent",
                                        ResultClass="LMI_ExtendedStaticIPAssignmentSettingData")
        self.assertEqual(len(assoc), 1)
        self.assertEqual(assoc[0]["ProtocolIFType"], 4096)

        # Add Stateless IPv6Type
        self.settingData["IPv6Type"] = pywbem.Uint16(9) # Stateless
        self.wbemconnection.ModifyInstance(self.settingData)
        # Check the changed value
        self.settingData = self.wbemconnection.GetInstance(self.settingData.path)
        self.assertEqual(self.settingData["IPv6Type"], pywbem.Uint16(9))
        self.assertEqual(self.settingData["ProtocolIFType"], pywbem.Uint16(32768))
        # Check association
        assoc = self.wbemconnection.Associators(self.settingData.path, AssocClass="LMI_OrderedIPAssignmentComponent",
                                        ResultClass="LMI_IPAssignmentSettingData")
        self.assertEqual(len(assoc), 1)
        self.assertEqual(assoc[0]["ProtocolIFType"], 4097)


        # TODO: get rid of it once the bug gets fixed
        self.skipTest("The rest of the test is skipped due to bug #1018781")


        # Disable both IPv4 and IPv6
        self.settingData["IPv4Type"] = pywbem.Uint16(0) # Disabled
        self.settingData["IPv6Type"] = pywbem.Uint16(0) # Disabled
        self.wbemconnection.ModifyInstance(self.settingData)
        # Check the changed value

        self.settingData = self.wbemconnection.GetInstance(self.settingData.path)
        self.assertEqual(self.settingData["IPv4Type"], pywbem.Uint16(0))
        self.assertEqual(self.settingData["IPv6Type"], pywbem.Uint16(0))
        assoc = self.wbemconnection.Associators(AssocClass="LMI_OrderedIPAssignmentComponent",
                                        ResultClass="LMI_IPAssignmentSettingData")
        self.assertEqual(len(assoc), 0)



#!/usr/bin/python
# -*- Coding:utf-8 -*-
#
# Copyright (C) 2012 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Radek Novacek <rnovacek@redhat.com>
#
# This test checks if the networking provider is compatible with parts of SMASH
# standard
#
from test_base import TestBase

import socket

ComputerSystemClass = "PG_ComputerSystem"

def IPv4_valid(ip):
    try:
        socket.inet_pton(socket.AF_INET, ip)
        return True
    except socket.error:
        return False

def IPv6_valid(ip):
    try:
        socket.inet_pton(socket.AF_INET6, ip)
        return True
    except socket.error:
        return False

class TestHostLANNetworkPortProfile(TestBase):
    """
    This test checks conformity against DSP1035 (version 1.0.2 - 2011-06-30).
    """
    def setUp(self):
        TestBase.setUp(self)

    def test_network_port_state_management(self):
        """ Section 7.1 of DSP 1035 """

        ports = self.wbemconnection.EnumerateInstances("LMI_EthernetPort")

        for port in ports:
            # 7.1.1 - Enabled but Offline
            # Not implemented - don't know how to emulate unplugged cable

            # 7.1.2 - Network Port State Management Is Supported - not our case

            # 7.1.3 - Network Port State Management Is Not Supported
            # 7.1.3.1 CIM_EnabledLogicalElementCapabilities
            capabilities = self.wbemconnection.Associators(port.path,
                    ResultClass="LMI_NetworkEnabledLogicalElementCapabilities",
                    AssocClass="LMI_NetworkElementCapabilities")
            self.assertEqual(len(capabilities), 1)
            # 7.1.3.1.1 - CIM_EnabledLogicalElementCapabilities.RequestedStatesSupported
            self.assertEqual(len(capabilities[0]["RequestedStatesSupported"]), 0,
                    "RequestedStatesSupported property of LMI_NetworkEnabledLogicalElementCapabilities should be empty list, is %s" % str(capabilities[0]["RequestedStatesSupported"]))
            # 7.1.3.2 - CIM_NetworkPort.RequestedState
            self.assertEqual(port["RequestedState"], 12) # Not Applicable
            # 7.1.3.3 - CIM_NetworkPort.EnabledState
            self.assertIn(port["EnabledState"], [2, 3, 5, 6])

            # 7.1.4 - Modifying ElementName Is Supported - Conditional - not our case

            # 7.1.5 - Modifying ElementName Is Not Supported
            # 7.1.5.1 - CIM_EnabledLogicalElementCapabilities
            # 7.1.5.1.1 - CIM_EnabledLogicalElementCapabilities.ElementNameEditSupported
            self.assertTrue(not capabilities[0]["ElementNameEditSupported"])

            # 7.1.6 - Representing the Physical Packaging - not implemented

    def test_representing_communication_endpoint(self):
        """ Section 7.2 of DSP 1035 """

        ports = self.wbemconnection.EnumerateInstances("LMI_EthernetPort")

        for port in ports:
            # 7.2.1 - Endpoint Identified by Hardware MAC
            # There shall be exactly one instance of CIM_LANEndpoint in which
            # the MACAddress property has the same value as the PermanentAddress
            # property of the associated CIM_NetworkPort instance.
            # 7.2.2 is almost the same
            lanEndpoints = self.wbemconnection.Associators(port.path,
                    ResultClass="LMI_LANEndpoint")
            self.assertGreater(len(lanEndpoints), 0)
            if port["PermanentAddress"] is not None and port["PermanentAddress"] != "":
                for lanEndpoint in lanEndpoints:
                    if lanEndpoint["MACAddress"] == port["PermanentAddress"]:
                        break
                else:
                    self.fail("No LANEndpoint with MACAddress same as NetworkPort's PermanentAddress (%s)" % port["DeviceID"])

        for lan in self.wbemconnection.EnumerateInstances("LMI_LANEndpoint"):
            # 7.2.3 - Relationship between the Interface and Port
            ports = self.wbemconnection.Associators(lan.path,
                    ResultClass="LMI_EthernetPort",
                    AssocClass="LMI_NetworkDeviceSAPImplementation")
            self.assertEqual(len(ports), 1)

            # 7.2.4 - Endpoint State Management Is Supported - Conditional
            # 7.2.4.1 - CIM_EnabledLogicalElementCapabilities
            capabilities = self.wbemconnection.Associators(lan.path,
                    ResultClass="LMI_NetworkEnabledLogicalElementCapabilities",
                    AssocClass="LMI_NetworkElementCapabilities")
            self.assertEqual(len(capabilities), 1)
            # 7.2.4.1.1 - CIM_EnabledLogicalElementCapabilities.RequestedStatesSupported
            self.assertListEqual(capabilities[0]["RequestedStatesSupported"], [2, 3])

            # 7.2.4.2 - CIM_LANEndpoint.RequestedState
            self.assertIn(lan["RequestedState"], capabilities[0]["RequestedStatesSupported"] + [5])

            # 7.2.4.3 - CIM_LANEndpoint.EnabledState
            self.assertIn(lan["EnabledState"], [2, 3])

            # 7.2.5 - Endpoint State Management Is Not Supported  - not our case
            # 7.2.6 - Modifying ElementName Is Supported - Conditional - not our case
            # 7.2.7 - Modifying ElementName Is Not Supported
            # 7.2.7.1 - CIM_EnabledLogicalElementCapabilities
            # 7.2.7.1.1 - CIM_EnabledLogicalElementCapabilities.ElementNameEditSupported
            self.assertTrue(not capabilities[0]["ElementNameEditSupported"])

    def test_managing_network_endpoints(self):
        """ Section 7.3 of DSP 1035 """
        # Creating network endpoints is not supported
        self.assertTrue("LMI_NetworkPortConfigurationService" not in self.wbemconnection.EnumerateClassNames(DeepInheritance=True))

    def test_representing_multiple_ports_controlled_from_a_single_controller(self):
        """ Section 7.4 of DSP 1035 """
        # Multiple ports controlled from single controller is not supported
        self.assertTrue("LMI_PortController" not in self.wbemconnection.EnumerateClassNames(DeepInheritance=True))

class TestIPConfigurationProfile(TestBase):
    """
    This test checks conformity against DSP1116 (version 1.0.0).
    """
    def assertAssociatedWithComputerSystem(self, cls, AssocClass):
        assocs = self.wbemconnection.Associators(cls.path, AssocClass=AssocClass, ResultClass=ComputerSystemClass)
        self.assertEqual(len(assocs), 1, "Instance of class %s should be associated with %s via %s" % (cls.classname, ComputerSystemClass, AssocClass))


    def assertAssociationCount(self, cls, count=1, **kw):
        assoc = self.wbemconnection.Associators(cls.path, **kw)

        self.assertEqual(len(assoc), count,
            "Instance of %s should be associated to %d instance(s) of %s via %s. %d associations found" %
            (cls.classname, count, kw.get("ResultClass", "<any>"), kw.get("AssocClass", "<any>"), len(assoc)))
        return assoc

    def test_representing_network_connection(self):
        """
        Representing the network connection.

        Section 7.1 of DSP 1116
        """

        # 7.1.1 - CIM_IPNetworkConnection
        ports = self.wbemconnection.EnumerateInstances("LMI_IPNetworkConnection")
        for port in ports:
            self.assertAssociatedWithComputerSystem(port,
                    AssocClass="LMI_NetworkHostedAccessPoint")

        # 7.1.2 Managing the CIM_IPNetworkConnection state
        for port in ports:
            self.assertIn(port["EnabledState"], [2, 3, 5, 6]) # Enabled, Disabled, Not Applicable, Enabled but Offline
            self.assertEqual(port["RequestedState"], 12) # Not Applicable

    def test_representing_ip_version(self):
        """
        Representing the IP version

        Section 7.2 of DSP 1116
        """

        ports = self.wbemconnection.EnumerateInstances("LMI_IPNetworkConnection")
        # 7.2.1 - CIM_IPVersionSettingData
        settingDatas = self.wbemconnection.EnumerateInstances("LMI_IPVersionSettingData")
        self.assertGreater(len(settingDatas), 0)
        for settingData in settingDatas:
            self.assertAssociatedWithComputerSystem(settingData,
                    AssocClass="LMI_IPVersionElementSettingData")

            self.assertAssociationCount(settingData, ResultClass="LMI_IPNetworkConnection", AssocClass="LMI_IPVersionElementSettingData", count=len(ports))

            # 7.2.1.1 - CIM_IPVersionSettingData.ProtocolIFType
            self.assertIn(settingData["ProtocolIFType"], [4096, 4097]) # IPv4 or IPv6

    def test_ip_setting(self):
        """
        IP setting

        Section 7.3 of DSP 1116
        """

        settingDatas = self.wbemconnection.EnumerateInstances("LMI_IPAssignmentSettingData")
        for settingData in settingDatas:
            # Static, DHCP, BOOTP, IPv4 Link Local, DHCPv6, IPv6AutoConfig, Stateless, Link Local, cumulative configuration
            self.assertIn(settingData["AddressOrigin"], [3, 4, 5, 6, 7, 8, 9, 10, 11])

            # 7.3.1 - CIM_IPAssignmentSettingData requirements for accumulation of
            #         settings, stateless IP assignment settings

            # 7.3.1.2 - CIM_IPAssignmentSettingData.ProtocolIFType
            if settingData["AddressOrigin"] == 9: # Stateless
                self.assertEqual(settingData["ProtocolIFType"], 4097) # IPv6


            # 7.3.1.3 - CIM_IPAssignmentSettingData.ConfigurationName
            if settingData["AddressOrigin"] == 11: # cumulative configuration
                self.assertTrue(settingData["ConfigurationName"] != "")

            # 7.3.2 - CIM_ExtendedStaticIPAssignmentSettingData requirements
            #         for static IP assignment settings
            if settingData.classname == "LMI_ExtendedStaticIPAssignmentSettingData":
                # 7.3.2.1 - CIM_ExtendedStaticIPAssignmentSettingData.AddressOrigin
                self.assertEqual(settingData["AddressOrigin"], 3) # Static
                # 7.3.2.2 - CIM_ExtendedStaticIPAssignmentSettingData.ProtocolIFType
                self.assertIn(settingData["ProtocolIFType"], [4096, 4097]) # IPv4 or IPv6
                # 7.3.2.3 - CIM_ExtendedStaticIPAssignmentSettingData.IPAddresses
                for ip in settingData["IPAddresses"]:
                    if settingData["ProtocolIFType"] == 4096:
                        self.assertTrue(IPv4_valid(ip), "IPv4 address is not valid")
                    else:
                        self.assertTrue(IPv6_valid(ip), "IPv6 address is not valid")
                # 7.3.2.4 - CIM_ExtendedStaticIPAssignmentSettingData.IPv6SubnetPrefixLengths
                if settingData["ProtocolIFType"] == 4097:
                    self.assertEqual(len(settingData["IPAddresses"]), len(settingData["IPv6SubnetPrefixLengths"]))
                else:
                    self.assertIsNone(settingData["IPv6SubnetPrefixLengths"])
                # 7.3.2.5 - CIM_ExtendedStaticIPAssignmentSettingData.SubnetMasks
                if settingData["ProtocolIFType"] == 4096:
                    self.assertEqual(len(settingData["IPAddresses"]), len(settingData["SubnetMasks"]))
                else:
                    self.assertIsNone(settingData["SubnetMasks"])
                # 7.3.2.6 - CIM_ExtendedStaticIPAssignmentSettingData.GatewayAddresses
                self.assertEqual(len(settingData["IPAddresses"]), len(settingData["GatewayAddresses"]))
                for ip in settingData["GatewayAddresses"]:
                    if settingData["ProtocolIFType"] == 4096:
                        self.assertTrue(IPv4_valid(ip), "IPv4 gateway is not valid")
                    else:
                        self.assertTrue(IPv6_valid(ip), "IPv6 gateway is not valid")

            # 7.3.3 - CIM_DHCPSettingData requirements for dynamic IP assignment settings
            if settingData.classname == "LMI_DHCPSettingData":
                # 7.3.3.1 - CIM_DHCPSettingData.AddressOrigin
                self.assertIn(settingData["AddressOrigin"], [4, 7]) # DHCP or DHCPv6
                # 7.3.3.2 - CIM_DHCPSettingData.ProtocolIFType
                if settingData["AddressOrigin"] == 4: # IPv4
                    self.assertEqual(settingData["ProtocolIFType"], 4096)
                else:
                    self.assertEqual(settingData["ProtocolIFType"], 4097)

    def test_representation_of_current_and_pending_settings(self):
        """
        Representation of current and pending settings.

        Section 7.4 of DSP1116
        """
        # 7.4.1 - CIM_ElementSettingData
        elementSettingDatas = self.wbemconnection.EnumerateInstances("LMI_IPElementSettingData")
        for elementSettingData in elementSettingDatas:
            # 7.4.1.1 - CIM_ElementSettingData.IsCurrent
            self.assertIn(elementSettingData["IsCurrent"], [1, 2]) # Is Current or Is Not Current
            # 7.4.1.2 - CIM_ElementSettingData.IsNext
            self.assertIn(elementSettingData["IsNext"], [1, 3, 2]) # Is Next or Is Next for Single user or Is Not Next
        # 7.4.2 - Modification of CIM_SettingData - not testable

    def test_representation_settings_of_network_connection(self):
        """
        Representation settings of a network connection

        Section 7.5 of DSP1116
        """
        # Concurrent settings and accumulation of settings are not testable here,
        # because we don't know which setting is concurrent (accumulation).
        pass

    def test_representing_ip_interface(self):
        """
        Representing the IP interface

        Section 7.6 of DSP1116
        """
        # 7.6.1 - CIM_IPProtocolEndpoint
        endpoints = self.wbemconnection.EnumerateInstances("LMI_IPProtocolEndpoint")
        for endpoint in endpoints:
            self.assertAssociatedWithComputerSystem(endpoint,
                    AssocClass="LMI_NetworkHostedAccessPoint")
            self.assertAssociationCount(endpoint, ResultClass="LMI_IPNetworkConnection", AssocClass="LMI_NetworkSAPSAPDependency")
            # 7.6.1.1 - CIM_IPProtocolEndpoint.AddressOrigin
            # TODO: get rid of 0, shouldn't be there
            self.assertIn(endpoint["AddressOrigin"], [0, 3, 4, 7, 9, 10]) # Static, DHCPv4, DHCPv6, Stateless, Link Local
            # 7.6.1.2 - CIM_IPProtocolEndpoint.ProtocolIFType
            self.assertIn(endpoint["ProtocolIFType"], [4096, 4097]) # IPv4 or IPv6
            if endpoint["ProtocolIFType"] == 4096:
                self.assertTrue(endpoint["IPv4Address"] is not None)
                self.assertTrue(endpoint["SubnetMask"] is not None)
            else:
                self.assertTrue(endpoint["IPv6Address"] is not None)
                self.assertTrue(endpoint["IPv6SubnetPrefixLength"] is not None)

            # 7.6.1.3 - CIM_IPProtocolEndpoint.IPv4Address
            if endpoint["ProtocolIFType"] == 4096:
                self.assertTrue(IPv4_valid(endpoint["IPv4Address"]), "IPv4 address is not valid")
            else:
                self.assertIsNone(endpoint["IPv4Address"])
            # 7.6.1.4 - CIM_IPProtocolEndpoint.SubnetMask
            if endpoint["ProtocolIFType"] == 4096:
                self.assertTrue(IPv4_valid(endpoint["SubnetMask"]), "IPv4 subnet mask is not valid")
            else:
                self.assertIsNone(endpoint["SubnetMask"])
            # 7.6.1.5 - CIM_IPProtocolEndpoint.IPv6Address
            if endpoint["ProtocolIFType"] == 4097:
                self.assertTrue(IPv6_valid(endpoint["IPv6Address"]), "IPv6 address is not valid")
            else:
                self.assertIsNone(endpoint["IPv6Address"])
            # 7.6.1.6 - CIM_IPProtocolEndpoint.IPv6SubnetPrefixLength
            if endpoint["ProtocolIFType"] == 4097:
                self.assertTrue(endpoint["IPv6SubnetPrefixLength"] is not None, "IPv6 prefix length is not valid")
            else:
                self.assertIsNone(endpoint["IPv6SubnetPrefixLength"])

    def test_ip_configuration_management(self):
        """
        IP configuration management

        Section 7.7 of DSP1116
        """
        # 7.7.1 - Configuration management is supported (optional)
        confServices = self.wbemconnection.EnumerateInstances("LMI_IPConfigurationService")
        self.assertEqual(len(confServices), 1)
        confService = confServices[0]

        self.assertAssociatedWithComputerSystem(confService,
                AssocClass="LMI_HostedIPConfigurationService")

        for networkConnection in self.wbemconnection.EnumerateInstances("LMI_IPNetworkConnection"):
            self.assertAssociationCount(networkConnection, ResultClass="LMI_IPConfigurationService", AssocClass="LMI_IPConfigurationServiceAffectsElement")

    def test_dhcp_client(self):
        """
        DHCP client

        Section 7.8 of DSP1116
        """
        # not yet implemented

    def test_dns_client_and_configuration(self):
        """
        DNS client and configuration

        Section 7.9 of DSP1116
        """
        # not yet implemented

    def test_relationship_with_network_interface(self):
        """
        Relationship with a network interface

        Section 7.10 of DSP1116
        """
        for networkConnection in self.wbemconnection.EnumerateInstances("LMI_IPNetworkConnection"):
            self.assertAssociationCount(networkConnection, ResultClass="LMI_LANEndpoint", AssocClass="LMI_EndpointForIPNetworkConnection")
        for protocolEndpoint in self.wbemconnection.EnumerateInstances("LMI_IPProtocolEndpoint"):
            self.assertAssociationCount(protocolEndpoint, ResultClass="LMI_LANEndpoint", AssocClass="LMI_BindsToLANEndpoint")

    def test_remote_services(self):
        """
        Remote services

        Section 7.11 of DSP1116
        """
        for rsap in self.wbemconnection.EnumerateInstances("LMI_NetworkRemoteServiceAccessPoint"):
            # 7.11.1 - Default gateway
            # 7.11.1.1 - CIM_RemoteServiceAccessPoint
            # 7.11.1.1.1 - CIM_RemoteServiceAccessPoint.AccessContext
            if rsap["AccessContext"] == 2: # Default Gateway
                # 7.11.1.1.2 - CIM_RemoteServiceAccessPoint.AccessInfo
                self.assertTrue(IPv4_valid(rsap["AccessInfo"]) or IPv6_valid(rsap["AccessInfo"]), "Invalid format of default gateway")
                # 7.11.1.2 - CIM_RemoteAccessAvailableToElement
                self.assertAssociationCount(rsap, ResultClass="LMI_IPNetworkConnection", AssocClass="LMI_NetworkRemoteAccessAvailableToElement")
                self.assertAssociatedWithComputerSystem(rsap,
                        AssocClass="LMI_NetworkRemoteAccessAvailableToElement")

            # 7.11.2 DHCP servers - not yet implemented
            # 7.11.3 DNS servers - not yet implemented
            """
            # 7.11.3.1 - CIM_RemoteServiceAccessPoint
            # 7.11.3.1.1 - CIM_RemoteServiceAccessPoint.AccessContext
            elif rsap["AccessContext"] == 3: # DNS Server
                # 7.11.3.1.2 - CIM_RemoteServiceAccessPoint.AccessInfo
                self.assertTrue(IPv4_valid(rsap["AccessInfo"]) or IPv6_valid(rsap["AccessInfo"]), "Invalid format of dns server")
                # 7.11.3.2 - CIM_RemoteAccessAvailableToElement
                assoc = self.wbemconnection.Associators(rsap.path,
                        ResultClass="LMI_IPNetworkConnection",
                        AssocClass="LMI_RemoteAccessAvailableToElement")
                self.assertEqual(len(assoc), 1)
                assoc = self.wbemconnection.Associators(rsap.path,
                        ResultClass=ComputerSystemClass,
                        AssocClass="LMI_RemoteAccessAvailableToElement")
                self.assertEqual(len(assoc), 1)
            """



#!/usr/bin/python
# -*- Coding:utf-8 -*-
#
# Copyright (C) 2012 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Radek Novacek <rnovacek@redhat.com>
#
# Base class for all tests

import os
import sys
if sys.version_info[0] > 2 or sys.version_info[1] > 6:
    import unittest
else:
    import unittest2 as unittest
import pywbem
import Queue
import socket
import threading
import BaseHTTPServer
import random

def get_interop_namespace():
    """
    Get name of interop namespace.
    """
    cimom = os.environ.get("LMI_CIMOM_BROKER", "tog-pegasus")
    if cimom != "tog-pegasus":
        # We always use interop with sfcbd
        return "root/interop"
    if os.path.isdir("/var/lib/Pegasus/repository/root#PG_InterOp"):
        return "root/PG_InterOp"
    else:
        return "root/interop"

class CIMListener(object):
    """ CIM Listener
    """
    class CIMHandler(BaseHTTPServer.BaseHTTPRequestHandler):
        def do_POST(self):
            data = self.rfile.read(int(self.headers['Content-Length']))
            tt = pywbem.parse_cim(pywbem.xml_to_tupletree(data))
            # Get the instance from CIM-XML, copied from
            # http://sf.net/apps/mediawiki/pywbem/?title=Indications_Tutorial
            insts = [x[1] for x in tt[2][2][0][2][2]]
            for inst in insts:
                self.callback(inst)
            self.send_response(200)
            self.end_headers()

        def log_message(self, format, *p):
            # suppress log messages
            pass

    def __init__(self, callback, http_port=5988):
        self.address = ('', http_port)
        self.CIMHandler.callback = callback
        self.thread = None
        self.server = None

    def start(self):
        BaseHTTPServer.HTTPServer.allow_reuse_address = True
        self.server = BaseHTTPServer.HTTPServer(self.address, self.CIMHandler)
        self.thread = threading.Thread(target=self.server.serve_forever)
        self.thread.start()

    def stop(self):
        if self.server is not None:
            self.server.shutdown()
            self.server.socket.close()
        if self.thread is not None:
            self.thread.join()

    def running(self):
        return self.thread is not None

class TestBase(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.url = os.environ.get("LMI_CIMOM_URL", "https://localhost:5989")
        cls.username = os.environ.get("LMI_CIMOM_USERNAME", "root")
        cls.password = os.environ.get("LMI_CIMOM_PASSWORD", "")

        cls.bus_name = os.environ.get("LMI_NETWORKING_BUS_NAME", "org.freedesktop.NetworkManager")
        cls.ports = os.environ.get("LMI_NETWORKING_PORTS", "").split(",")
        cls.port = cls.ports[0]
        cls.fake_test = os.environ.get("LMI_NETWORKING_FAKE_NM", "0") == "1"

        if cls.url == "local://pegasus":
            # Special string for local pegasus connection
            cls.wbemconnection = pywbem.PegasusUDSConnection()
        elif cls.url == "local://sfcbd":
            # Special string for local sfcbd connection
            cls.wbemconnection = pywbem.SFCBUDSConnection()
        else:
            cls.wbemconnection = pywbem.WBEMConnection(cls.url, (cls.username, cls.password))

        cls.indication_port = random.randint(12000, 13000)
        cls.indication_queue = Queue.Queue()
        cls.listener = CIMListener(
                callback=cls._process_indication,
                http_port=cls.indication_port)

    @classmethod
    def tearDownClass(cls):
        # stop listening
        cls.listener.stop()

    def setUp(self):
        self.subscribed = {}

    def tearDown(self):
        # put the class to initial state
        # unsubscribe from indications
        if self.subscribed:
            for name in self.subscribed.keys():
                self.unsubscribe(name)

    def subscribe(self, filter_name, query=None, querylang="DMTF:CQL"):
        """
        Create indication subscription for given filter name.
        """
        namespace = get_interop_namespace()
        hostname = socket.gethostname()

        if query is not None:
            # Create filter first
            filterinst = pywbem.CIMInstance('CIM_IndicationFilter')
            filterinst['CreationClassName'] = 'CIM_IndicationFilter'
            filterinst['SystemCreationClassName'] = 'CIM_ComputerSystem'
            filterinst['SystemName'] = hostname
            filterinst['Name'] = filter_name
            filterinst['Query'] = query
            filterinst['QueryLanguage'] = querylang
            filterinst['SourceNamespace'] = "root/cimv2"
            cop = pywbem.CIMInstanceName('CIM_IndicationFilter')
            cop.keybindings = { 'CreationClassName': 'CIM_IndicationFilter',
                'SystemClassName': 'CIM_ComputerSystem',
                'SystemName': hostname,
                'Name': filter_name
            }
            cop.namespace=namespace
            filterinst.path = cop
            indfilter = self.wbemconnection.CreateInstance(filterinst)
        else:
            # the filter is already created, assemble its name
            indfilter = pywbem.CIMInstanceName(
                    classname="CIM_IndicationFilter",
                    namespace=namespace,
                    keybindings={
                        'CreationClassName': 'CIM_IndicationFilter',
                        'SystemClassName': 'CIM_ComputerSystem',
                        'SystemName': hostname,
                        'Name': filter_name})

        # create destination
        destinst = pywbem.CIMInstance('CIM_ListenerDestinationCIMXML')
        destinst['CreationClassName'] = 'CIM_ListenerDestinationCIMXML'
        destinst['SystemCreationClassName'] = 'CIM_ComputerSystem'
        destinst['SystemName'] = hostname
        destinst['Name'] = filter_name
        destinst['Destination'] = "http://localhost:%d" % (self.indication_port)
        destinst['PersistenceType'] = pywbem.Uint16(3) # Transient
        cop = pywbem.CIMInstanceName('CIM_ListenerDestinationCIMXML')
        cop.keybindings = { 'CreationClassName':'CIM_ListenerDestinationCIMXML',
                'SystemClassName':'CIM_ComputerSystem',
                'SystemName':hostname,
                'Name':filter_name }
        cop.namespace = namespace
        destinst.path = cop
        destname = self.wbemconnection.CreateInstance(destinst)

        # create the subscription
        subinst = pywbem.CIMInstance('CIM_IndicationSubscription')
        subinst['Filter'] = indfilter
        subinst['Handler'] = destname
        cop = pywbem.CIMInstanceName('CIM_IndicationSubscription')
        cop.keybindings = { 'Filter': indfilter,
                'Handler': destname }
        cop.namespace = namespace
        subinst.path = cop
        subscription = self.wbemconnection.CreateInstance(subinst)

        self.subscribed[filter_name] = [subscription, destname]

        # start listening
        if not self.listener.running():
            self._start_listening()
        return subscription

    def unsubscribe(self, filter_name):
        """
        Unsubscribe fron given filter.
        """
        _list = self.subscribed.pop(filter_name)
        for instance in _list:
            self.wbemconnection.DeleteInstance(instance)

    def _start_listening(self):
        """ Start listening for incoming indications. """
        self.listener.start()

    @classmethod
    def _process_indication(cls, indication):
        """ Callback to process one indication."""
        cls.indication_queue.put(indication)

    def get_indication(self, timeout):
        """ Wait for an indication for given nr. of seconds and return it."""
        try:
            indication = self.indication_queue.get(timeout=timeout)
        except Queue.Empty:
            raise AssertionError("Timeout when waiting for indicaiton")
        self.indication_queue.task_done()
        return indication

    def wait_for_job(self, job, timeout=10):
        # Job is started, wait for finish
        while 1:
            indication = self.get_indication(timeout)
            if indication["SourceInstance"].classname == "LMI_NetworkJob":
                if indication["SourceInstance"]["InstanceID"] == job["InstanceID"]:
                    return indication["SourceInstance"]

    def wait_for_job_completion(self, job):
        """
        Wait for incoming job changes indications. When we have JobState == 7,
        the job succeeds. Otherwise wait_for_job raises an exception that
        will propagate out of this method
        """
        while 1:
            if self.wait_for_job(job)["JobState"] == 7: # Completed
                return




import sys
from test_base import TestBase
import pywbem
import datetime
import time

class TestIndications(TestBase):
    def create_setting(self):
        self.ipNetworkConnection = None
        for networkConnection in self.wbemconnection.EnumerateInstances("LMI_IPNetworkConnection"):
            if networkConnection["Name"] == self.port:
                self.ipNetworkConnection = networkConnection
        self.assertTrue(self.ipNetworkConnection is not None)
        capas = self.wbemconnection.Associators(self.ipNetworkConnection.path, ResultClass="LMI_IPNetworkConnectionCapabilities")
        self.assertGreater(len(capas), 0)
        capa = capas[0]

        self.caption = "XXX Test Jobs"
        rc = self.wbemconnection.InvokeMethod("LMI_CreateIPSetting", capa.path,
                Caption=self.caption, Type=pywbem.Uint16(1),
                IPv4Type=pywbem.Uint16(4))
        self.assertEqual(len(rc), 2)
        self.assertEqual(rc[0], 0)
        self.addCleanup(self.wbemconnection.DeleteInstance, rc[1]["SettingData"])
        return self.wbemconnection.GetInstance(rc[1]["SettingData"])

    def setUp(self):
        TestBase.setUp(self)
        filter_name = "test_job_%d" % (time.time() * 1000)
        sub = self.subscribe(filter_name, "select * from LMI_NetworkInstModification where SourceInstance isa LMI_NetworkJob")

    def test_job(self):
        """ Test job creation and its properties """
        settingData = self.create_setting()

        conf_services = self.wbemconnection.EnumerateInstances("LMI_IPConfigurationService")
        self.assertEqual(len(conf_services), 1)
        conf_service = conf_services[0]

        rc = self.wbemconnection.InvokeMethod("ApplySettingToIPNetworkConnection",
                conf_service.path, SettingData=settingData.path,
                IPNetworkConnection=self.ipNetworkConnection.path,
                Mode=pywbem.Uint16(32768))
        self.assertGreater(len(rc), 1)
        self.assertEqual(rc[0], 4096) # Job started
        self.assertIn("Job", rc[1].keys())
        job = self.wbemconnection.GetInstance(rc[1]["Job"])

        self.assertEqual(job["DeleteOnCompletion"], True)
        self.assertTrue(job["TimeBeforeRemoval"].is_interval)
        self.assertEqual(job["TimeBeforeRemoval"].timedelta.seconds, 300)

        # Check LMI_OwningNetworkJobElement association
        assoc = self.wbemconnection.Associators(job.path, AssocClass="LMI_OwningNetworkJobElement")
        self.assertEqual(len(assoc), 1)
        self.assertEqual(assoc[0].classname, "LMI_IPConfigurationService")

        # Check LMI_AffectedNetworkJobElement association
        assoc = self.wbemconnection.Associators(job.path, AssocClass="LMI_AffectedNetworkJobElement")
        self.assertEqual(len(assoc), 2)
        for a in assoc:
            if a.classname == "LMI_IPAssignmentSettingData":
                self.assertEqual(a["Caption"], self.caption)
            elif a.classname == "LMI_IPNetworkConnection":
                self.assertEqual(a["Name"], self.port)
            else:
                self.assertTrue(False, "Invalid association to %s" % a.classname)

        # Wait for job completion
        self.wait_for_job(job)

        # Modify the job
        job["TimeBeforeRemoval"] = pywbem.CIMDateTime(datetime.timedelta(seconds=3))
        self.wbemconnection.ModifyInstance(job)

        # Check if the job is modified
        job = self.wbemconnection.GetInstance(job.path)
        self.assertTrue(job["TimeBeforeRemoval"].is_interval)
        self.assertEqual(job["TimeBeforeRemoval"].timedelta.seconds, 3)

        # Wait for job deletion
        time.sleep(5)

        # Check if the job was really deleted
        self.assertRaises(pywbem.CIMError, self.wbemconnection.GetInstance, job.path)



#!/usr/bin/python
# -*- Coding:utf-8 -*-
#
# Copyright (C) 2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Radek Novacek <rnovacek@redhat.com>
#
# This test tests applying connections in different modes.


import sys
import os
import pywbem
import time
import IPy

from test_base import TestBase, unittest

class TestApplyConnection(TestBase):
    @classmethod
    def setUpClass(cls):
        TestBase.setUpClass()
        # Get reference on port
        cls.ports = cls.wbemconnection.ExecQuery("WQL", 'select * from LMI_IPNetworkConnection where Name = "%s"' % cls.port)
        if len(cls.ports) < 1:
            raise AssertionError("No LMI_IPNetworkConnection with Name %s" % cls.port)
        cls.portref = cls.ports[0]

        cls.confService = cls.wbemconnection.EnumerateInstances("LMI_IPConfigurationService")[0]

        capas = cls.wbemconnection.Associators(cls.portref.path, ResultClass="LMI_IPNetworkConnectionCapabilities")
        if len(capas) != 1:
            raise AssertionError("No LMI_IPNetworkConnectionCapabilities for port %s" % cls.portref["Name"])
        capa = capas[0]

        # Create a couple of connections
        rc = cls.wbemconnection.InvokeMethod("LMI_CreateIPSetting",
                capa.path, Caption="XXX Connection 1",
                IPv4Type=pywbem.Uint16(4))
        if rc[0] != 0:
            cls.skipTest("Unable to create connection - ApplyConnection test will be skipped")
            return
        cls.con1 = cls.wbemconnection.GetInstance(rc[1]["SettingData"])

        rc = cls.wbemconnection.InvokeMethod("LMI_CreateIPSetting",
                capa.path, Caption="XXX Connection 2",
                IPv4Type=pywbem.Uint16(4))
        if rc[0] != 0:
            cls.skipTest("Unable to create connection - ApplyConnection test will be skipped")
            return
        cls.con2 = cls.wbemconnection.GetInstance(rc[1]["SettingData"])

        rc = cls.wbemconnection.InvokeMethod("LMI_CreateIPSetting",
                capa.path, Caption="XXX Connection 3",
                IPv4Type=pywbem.Uint16(4))
        if rc[0] != 0:
            cls.skipTest("Unable to create connection - ApplyConnection test will be skipped")
            return
        cls.con3 = cls.wbemconnection.GetInstance(rc[1]["SettingData"])

    @classmethod
    def tearDownClass(cls):
        TestBase.tearDownClass()
        cls.wbemconnection.DeleteInstance(cls.con1.path)
        cls.wbemconnection.DeleteInstance(cls.con2.path)
        cls.wbemconnection.DeleteInstance(cls.con3.path)

    def setUp(self):
        TestBase.setUp(self)
        # Subscribe to indication
        self.filter_name = "test_apply_%d" % (time.time() * 1000)
        self.subscribe(self.filter_name, "select * from LMI_NetworkInstModification where SourceInstance isa LMI_NetworkJob")

    def assertIPElementSettingData(self, esd, isCurrent, isNext):
        portName = esd["ManagedElement"]["Name"]
        connectionName = self.wbemconnection.GetInstance(esd["SettingData"])["Caption"]
        if isCurrent is not None:
            self.assertEqual(esd["IsCurrent"], isCurrent,
                    "Value of IsCurrent should be %d (but it's %d) for port %s "
                    "and configuration %s" % (isCurrent, esd["IsCurrent"],
                    portName, connectionName))
        if isNext is not None:
            self.assertEqual(esd["IsNext"], isNext,
                    "Value of IsNext should be %d (but it's %d) for port %s "
                    "and configuration %s" % (isNext, esd["IsNext"],
                    portName, connectionName))

    def test_mode_1(self):
        """ Test activating connection with Mode == 1

        Mode 1: IsNext = 1 (Is Next), IsCurrent = 1 (Is Current)
        """
        rc = self.wbemconnection.InvokeMethod("ApplySettingToIPNetworkConnection",
                self.confService.path, SettingData=self.con1.path,
                IPNetworkConnection=self.portref.path, Mode=pywbem.Uint16(1))
        self.assertIn(rc[0], [0, 4096], "Applying setting failed")
        if rc[0] == 4096: # Job started
            self.wait_for_job_completion(rc[1]["Job"])

        # Test values of IsNext and IsCurrent for all created connections
        for esd in self.wbemconnection.References(self.con1.path, ResultClass="LMI_IPElementSettingData"):
            self.assertIPElementSettingData(esd, 1, 1)
        for esd in self.wbemconnection.References(self.con2.path, ResultClass="LMI_IPElementSettingData") + \
                   self.wbemconnection.References(self.con3.path, ResultClass="LMI_IPElementSettingData"):
            self.assertIPElementSettingData(esd, 2, 2)

    def test_mode_2(self):
        """ Test activating connection with Mode == 2

        Mode 2: IsNext = 1 (Is Next) and IsCurrent not affected
        """
        esd = self.wbemconnection.References(self.con1.path, ResultClass="LMI_IPElementSettingData")[0]
        old_isCurrent = esd["IsCurrent"]

        rc = self.wbemconnection.InvokeMethod("ApplySettingToIPNetworkConnection",
                self.confService.path, SettingData=self.con1.path,
                IPNetworkConnection=self.portref.path, Mode=pywbem.Uint16(2))
        self.assertIn(rc[0], [0, 4096], "Applying setting failed")
        if rc[0] == 4096: # Job started
            self.wait_for_job_completion(rc[1]["Job"])

        # Test values of IsNext and IsCurrent for all created connections
        for esd in self.wbemconnection.References(self.con1.path, ResultClass="LMI_IPElementSettingData"):
            self.assertIPElementSettingData(esd, old_isCurrent, 1)
        for esd in self.wbemconnection.References(self.con2.path, ResultClass="LMI_IPElementSettingData") + \
                   self.wbemconnection.References(self.con3.path, ResultClass="LMI_IPElementSettingData"):
            self.assertIPElementSettingData(esd, None, 2)

    def test_mode_4(self):
        """ Test activating connection with Mode == 4

        Mode 4 - IsNext = 2 (Is Not Next) and IsCurrent = 2 (Is Not Current)
        """
        rc = self.wbemconnection.InvokeMethod("ApplySettingToIPNetworkConnection",
                self.confService.path, SettingData=self.con1.path,
                IPNetworkConnection=self.portref.path, Mode=pywbem.Uint16(4))
        self.assertIn(rc[0], [0, 4096], "Applying setting failed")
        if rc[0] == 4096: # Job started
            self.wait_for_job_completion(rc[1]["Job"])

        # Test values of IsNext and IsCurrent for all created connections
        for esd in self.wbemconnection.References(self.con1.path, ResultClass="LMI_IPElementSettingData") + \
                   self.wbemconnection.References(self.con2.path, ResultClass="LMI_IPElementSettingData") + \
                   self.wbemconnection.References(self.con3.path, ResultClass="LMI_IPElementSettingData"):
            self.assertIPElementSettingData(esd, 2, 2)

    def test_mode_5(self):
        """ Test activating connection with Mode == 5

        Mode 5 - IsNext = 2 (Is Not Next) and IsCurrent not affected
        """
        esd = self.wbemconnection.References(self.con1.path, ResultClass="LMI_IPElementSettingData")[0]
        old_isCurrent = esd["IsCurrent"]

        rc = self.wbemconnection.InvokeMethod("ApplySettingToIPNetworkConnection",
                self.confService.path, SettingData=self.con1.path,
                IPNetworkConnection=self.portref.path, Mode=pywbem.Uint16(5))
        self.assertIn(rc[0], [0, 4096], "Applying setting failed")
        if rc[0] == 4096: # Job started
            self.wait_for_job_completion(rc[1]["Job"])

        # Test values of IsNext and IsCurrent for all created connections
        for esd in self.wbemconnection.References(self.con1.path, ResultClass="LMI_IPElementSettingData"):
            self.assertIPElementSettingData(esd, old_isCurrent, 2)
        for esd in self.wbemconnection.References(self.con2.path, ResultClass="LMI_IPElementSettingData") + \
                   self.wbemconnection.References(self.con3.path, ResultClass="LMI_IPElementSettingData"):
            self.assertIPElementSettingData(esd, 2, 2)

    def test_mode_32768(self):
        """ Test activating connection with Mode == 32768

        Mode 32768 - IsCurrent = 1 (Is Current) and IsNext not affected
        """
        esd = self.wbemconnection.References(self.con1.path, ResultClass="LMI_IPElementSettingData")[0]
        old_isNext = esd["IsNext"]

        rc = self.wbemconnection.InvokeMethod("ApplySettingToIPNetworkConnection",
                self.confService.path, SettingData=self.con1.path,
                IPNetworkConnection=self.portref.path, Mode=pywbem.Uint16(32768))
        self.assertIn(rc[0], [0, 4096], "Applying setting failed")
        if rc[0] == 4096: # Job started
            self.wait_for_job_completion(rc[1]["Job"])

        # Test values of IsNext and IsCurrent for all created connections
        for esd in self.wbemconnection.References(self.con1.path, ResultClass="LMI_IPElementSettingData"):
            self.assertIPElementSettingData(esd, 1, old_isNext)
        for esd in self.wbemconnection.References(self.con2.path, ResultClass="LMI_IPElementSettingData") + \
                   self.wbemconnection.References(self.con3.path, ResultClass="LMI_IPElementSettingData"):
            self.assertIPElementSettingData(esd, 2, 2)

    def test_mode_32769(self):
        """ Test activating connection with Mode == 32769

        Mode 32769 - ``IsCurrent = 2 (Is Not Current) and IsNext not affected
        """
        esd = self.wbemconnection.References(self.con1.path, ResultClass="LMI_IPElementSettingData")[0]
        old_isNext = esd["IsNext"]

        rc = self.wbemconnection.InvokeMethod("ApplySettingToIPNetworkConnection",
                self.confService.path, SettingData=self.con1.path,
                IPNetworkConnection=self.portref.path, Mode=pywbem.Uint16(32769))
        self.assertIn(rc[0], [0, 4096], "Applying setting failed")
        if rc[0] == 4096: # Job started
            self.wait_for_job_completion(rc[1]["Job"])

        # Test values of IsNext and IsCurrent for all created connections
        for esd in self.wbemconnection.References(self.con1.path, ResultClass="LMI_IPElementSettingData"):
            self.assertIPElementSettingData(esd, 2, old_isNext)
        for esd in self.wbemconnection.References(self.con2.path, ResultClass="LMI_IPElementSettingData") + \
                   self.wbemconnection.References(self.con3.path, ResultClass="LMI_IPElementSettingData"):
            self.assertIPElementSettingData(esd, 2, 2)

if __name__ == '__main__':
    import os
    os.environ["LMI_CIMOM_URL"] = "local://pegasus"
    unittest.main()
